@@ -16,7 +16,7 @@
  * @endcopyright
  *
  * @file svn_md5.h
- * @brief Code for converting and comparing MD5 checksums.
+ * @brief Converting and comparing MD5 checksums.
  */
 
 #ifndef SVN_MD5_H
@@ -34,23 +34,30 @@ extern "C" {
 
 
 /** The MD5 digest for the empty string. */
-const unsigned char *svn_md5_empty_string_digest (void);
+const unsigned char *svn_md5_empty_string_digest(void);
+
+
+/** Return the hex representation of @a digest, which must be
+ * @c APR_MD5_DIGESTSIZE bytes long, allocating the string in @a pool.
+ */
+const char *svn_md5_digest_to_cstring_display(const unsigned char digest[],
+                                              apr_pool_t *pool);
 
 
 /** Return the hex representation of @a digest, which must be
  * @c APR_MD5_DIGESTSIZE bytes long, allocating the string in @a pool.
  * If @a digest is all zeros, then return NULL.
  */
-const char *svn_md5_digest_to_cstring (const unsigned char digest[],
-                                       apr_pool_t *pool);
+const char *svn_md5_digest_to_cstring(const unsigned char digest[],
+                                      apr_pool_t *pool);
 
 
 /** Compare digests @a d1 and @a d2, each @c APR_MD5_DIGESTSIZE bytes long.
- * If neither is all zeros, and they do not match, then return false;
- * else return true.
+ * If neither is all zeros, and they do not match, then return FALSE;
+ * else return TRUE.
  */
-svn_boolean_t svn_md5_digests_match (unsigned const char d1[],
-                                     unsigned const char d2[]);
+svn_boolean_t svn_md5_digests_match(const unsigned char d1[],
+                                    const unsigned char d2[]);
 
 #ifdef __cplusplus
 }
