@@ -2,7 +2,7 @@
  * locking_commands.c:  Implementation of lock and unlock.
  *
  * ====================================================================
- * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -29,6 +29,7 @@
 #include "svn_pools.h"
 
 #include "svn_private_config.h"
+#include "private/svn_wc_private.h"
 
 
 /*** Code. ***/
@@ -54,8 +55,8 @@ struct lock_baton
  * than SVN_ERR_FS_LOCK_OWNER_MISMATCH.
  */
 static svn_error_t *
-store_locks_callback(void *baton, 
-                     const char *rel_url, 
+store_locks_callback(void *baton,
+                     const char *rel_url,
                      svn_boolean_t do_lock,
                      const svn_lock_t *lock,
                      svn_error_t *ra_err, apr_pool_t *pool)
@@ -82,7 +83,7 @@ store_locks_callback(void *baton,
     {
       char *path = apr_hash_get(lb->urls_to_paths, rel_url,
                                 APR_HASH_KEY_STRING);
-      abs_path = svn_path_join(svn_wc_adm_access_path(lb->adm_access), 
+      abs_path = svn_path_join(svn_wc_adm_access_path(lb->adm_access),
                                path, lb->pool);
 
       SVN_ERR(svn_wc_adm_probe_retrieve(&adm_access, lb->adm_access,
@@ -115,7 +116,7 @@ store_locks_callback(void *baton,
             notify->lock_state = svn_wc_notify_lock_state_unchanged;
         }
     }
-  
+
   if (lb->ctx->notify_func2)
     lb->ctx->notify_func2(lb->ctx->notify_baton2, notify, pool);
 
@@ -129,8 +130,8 @@ store_locks_callback(void *baton,
  * targets (as opposed to the common local path).
  *
  * If all the targets are local paths within the same wc, i.e., they
- * share a common parent at some level, set and *PARENT_ADM_ACCESS_P
- * to and adm_access of that common parent.  *PARENT_ADM_ACCESS_P will
+ * share a common parent at some level, set *PARENT_ADM_ACCESS_P
+ * to the adm_access of that common parent.  *PARENT_ADM_ACCESS_P will
  * be associated with adm_access objects for all the other paths,
  * which are locked in the working copy while we lock them in the
  * repository.
@@ -176,10 +177,10 @@ organize_lock_targets(const char **common_parent,
   apr_array_header_t *rel_targets = apr_array_make(pool, 1,
                                                    sizeof(const char *));
   apr_hash_t *rel_targets_ret = apr_hash_make(pool);
-  apr_pool_t *subpool = svn_pool_create(pool); 
+  apr_pool_t *subpool = svn_pool_create(pool);
 
   /* Get the common parent and all relative paths */
-  SVN_ERR(svn_path_condense_targets(common_parent, &rel_targets, targets, 
+  SVN_ERR(svn_path_condense_targets(common_parent, &rel_targets, targets,
                                     FALSE, pool));
 
   /* svn_path_condense_targets leaves paths empty if TARGETS only had
@@ -196,7 +197,7 @@ organize_lock_targets(const char **common_parent,
     return svn_error_create
       (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
        _("No common parent found, unable to operate on disjoint arguments"));
-  
+
   if (svn_path_is_url(*common_parent))
     {
       svn_revnum_t *invalid_revnum;
@@ -206,7 +207,7 @@ organize_lock_targets(const char **common_parent,
 
       for (i = 0; i < rel_targets->nelts; i++)
         {
-          const char *target = ((const char **) (rel_targets->elts))[i];
+          const char *target = APR_ARRAY_IDX(rel_targets, i, const char *);
           apr_hash_set(rel_targets_ret, svn_path_uri_decode(target, pool),
                        APR_HASH_KEY_STRING,
                        do_lock ? (const void *) invalid_revnum
@@ -216,7 +217,7 @@ organize_lock_targets(const char **common_parent,
     }
   else  /* common parent is a local path */
     {
-      int max_depth = 0;
+      int max_levels_to_lock = 0;
       apr_array_header_t *rel_urls;
       apr_array_header_t *urls = apr_array_make(pool, 1,
                                                 sizeof(const char *));
@@ -227,23 +228,24 @@ organize_lock_targets(const char **common_parent,
          is the depth to which we need to lock the WC. */
       for (i = 0; i < rel_targets->nelts; ++i)
         {
-          const char *target = ((const char **) (rel_targets->elts))[i];
+          const char *target = APR_ARRAY_IDX(rel_targets, i, const char *);
           int n = svn_path_component_count(target);
 
-          if (n > max_depth)
-            max_depth = n;
+          if (n > max_levels_to_lock)
+            max_levels_to_lock = n;
         }
 
       SVN_ERR(svn_wc_adm_probe_open3(parent_adm_access_p, NULL,
-                                     *common_parent, 
-                                     TRUE, max_depth, ctx->cancel_func, 
-                                     ctx->cancel_baton, pool));  
+                                     *common_parent,
+                                     TRUE, max_levels_to_lock,
+                                     ctx->cancel_func, ctx->cancel_baton,
+                                     pool));
 
       /* Get the url for each target and verify all paths. */
       for (i = 0; i < rel_targets->nelts; i++)
         {
           const svn_wc_entry_t *entry;
-          const char *target = ((const char **) (rel_targets->elts))[i];
+          const char *target = APR_ARRAY_IDX(rel_targets, i, const char *);
           const char *abs_path;
 
           svn_pool_clear(subpool);
@@ -251,24 +253,20 @@ organize_lock_targets(const char **common_parent,
           abs_path = svn_path_join
             (svn_wc_adm_access_path(*parent_adm_access_p), target, subpool);
 
-          SVN_ERR(svn_wc_entry(&entry, abs_path, *parent_adm_access_p, FALSE,
-                               subpool));
+          SVN_ERR(svn_wc__entry_versioned(&entry, abs_path,
+                                         *parent_adm_access_p, FALSE, subpool));
 
-          if (! entry)
-            return svn_error_createf(SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                                     _("'%s' is not under version control"), 
-                                     svn_path_local_style(target, pool));
           if (! entry->url)
             return svn_error_createf(SVN_ERR_ENTRY_MISSING_URL, NULL,
                                      _("'%s' has no URL"),
                                      svn_path_local_style(target, pool));
 
-          (*((const char **)(apr_array_push(urls)))) = apr_pstrdup 
-            (pool, entry->url);
+          APR_ARRAY_PUSH(urls, const char *) = apr_pstrdup(pool,
+                                                           entry->url);
         }
 
       /* Condense our absolute urls and get the relative urls. */
-      SVN_ERR(svn_path_condense_targets(&common_url, &rel_urls, urls, 
+      SVN_ERR(svn_path_condense_targets(&common_url, &rel_urls, urls,
                                         FALSE, pool));
 
       /* svn_path_condense_targets leaves paths empty if TARGETS only had
@@ -279,7 +277,7 @@ organize_lock_targets(const char **common_parent,
           common_url = svn_path_dirname(common_url, pool);
           APR_ARRAY_PUSH(rel_urls, char *) = base_name;
         }
-      
+
       /* If we have no common URL parent, bail (cross-repos lock attempt) */
       if (common_url == NULL || (common_url)[0] == '\0')
         return svn_error_create
@@ -296,10 +294,10 @@ organize_lock_targets(const char **common_parent,
           const char *abs_path;
           const char *decoded_url = svn_path_uri_decode(url, pool);
 
-          svn_pool_clear(subpool); 
+          svn_pool_clear(subpool);
 
           apr_hash_set(urls_hash, decoded_url,
-                       APR_HASH_KEY_STRING, 
+                       APR_HASH_KEY_STRING,
                        apr_pstrdup(pool, target));
 
           abs_path = svn_path_join
@@ -313,7 +311,7 @@ organize_lock_targets(const char **common_parent,
               svn_revnum_t *revnum;
               revnum = apr_palloc(pool, sizeof(* revnum));
               *revnum = entry->revision;
-              
+
               apr_hash_set(rel_targets_ret, decoded_url,
                            APR_HASH_KEY_STRING, revnum);
             }
@@ -323,12 +321,12 @@ organize_lock_targets(const char **common_parent,
               if (! force)
                 {
                   if (! entry->lock_token)
-                    return svn_error_createf 
+                    return svn_error_createf
                       (SVN_ERR_CLIENT_MISSING_LOCK_TOKEN, NULL,
                        _("'%s' is not locked in this working copy"), target);
-                  
+
                   apr_hash_set(rel_targets_ret, decoded_url,
-                               APR_HASH_KEY_STRING, 
+                               APR_HASH_KEY_STRING,
                                apr_pstrdup(pool, entry->lock_token));
                 }
               else
@@ -343,9 +341,9 @@ organize_lock_targets(const char **common_parent,
       *rel_fs_paths_p = urls_hash;
       *common_parent = common_url;
     }
-  
+
   *rel_targets_p = rel_targets_ret;
-  svn_pool_destroy(subpool);  
+  svn_pool_destroy(subpool);
   return SVN_NO_ERROR;
 }
 
@@ -371,7 +369,7 @@ fetch_tokens(svn_ra_session_t *ra_session, apr_hash_t *path_tokens,
       SVN_ERR(svn_ra_get_lock(ra_session, &lock, path, iterpool));
 
       if (! lock)
-        return svn_error_createf 
+        return svn_error_createf
           (SVN_ERR_CLIENT_MISSING_LOCK_TOKEN, NULL,
            _("'%s' is not locked"), path);
 
@@ -406,11 +404,11 @@ svn_client_lock(const apr_array_header_t *targets,
       if (! svn_xml_is_xml_safe(comment, strlen(comment)))
         return svn_error_create
           (SVN_ERR_XML_UNESCAPABLE_DATA, NULL,
-           _("Lock comment has illegal characters"));      
+           _("Lock comment contains illegal characters"));
     }
 
   SVN_ERR(organize_lock_targets(&common_parent, &adm_access,
-                                &path_revs, &urls_to_paths, targets, TRUE, 
+                                &path_revs, &urls_to_paths, targets, TRUE,
                                 steal_lock, ctx, pool));
 
   /* Open an RA session to the common parent of TARGETS. */
@@ -425,7 +423,7 @@ svn_client_lock(const apr_array_header_t *targets,
   cb.ctx = ctx;
 
   /* Lock the paths. */
-  SVN_ERR(svn_ra_lock(ra_session, path_revs, comment, 
+  SVN_ERR(svn_ra_lock(ra_session, path_revs, comment,
                       steal_lock, store_locks_callback, &cb, pool));
 
   /* Unlock the wc. */
@@ -451,7 +449,7 @@ svn_client_unlock(const apr_array_header_t *targets,
     return SVN_NO_ERROR;
 
   SVN_ERR(organize_lock_targets(&common_parent, &adm_access,
-                                &path_tokens, &urls_to_paths, targets, 
+                                &path_tokens, &urls_to_paths, targets,
                                 FALSE, break_lock, ctx, pool));
 
   /* Open an RA session. */
@@ -473,7 +471,7 @@ svn_client_unlock(const apr_array_header_t *targets,
   cb.ctx = ctx;
 
   /* Unlock the paths. */
-  SVN_ERR(svn_ra_unlock(ra_session, path_tokens, break_lock, 
+  SVN_ERR(svn_ra_unlock(ra_session, path_tokens, break_lock,
                         store_locks_callback, &cb, pool));
 
   /* Unlock the wc. */
