@@ -50,11 +50,11 @@ svn_fs_bdb__open_nodes_table(DB **nodes_p,
 
   BDB_ERR(svn_fs_bdb__check_version());
   BDB_ERR(db_create(&nodes, env, 0));
-  BDB_ERR(nodes->open(SVN_BDB_OPEN_PARAMS(nodes, NULL),
-                      "nodes", 0, DB_BTREE,
-                      open_flags, 0666));
+  BDB_ERR((nodes->open)(SVN_BDB_OPEN_PARAMS(nodes, NULL),
+                        "nodes", 0, DB_BTREE,
+                        open_flags, 0666));
 
-  /* Create the `next-id' table entry (use '1' because '0' is
+  /* Create the `next-key' table entry (use '1' because '0' is
      reserved for the root directory to use). */
   if (create)
   {
@@ -137,7 +137,7 @@ svn_fs_bdb__new_successor_id(svn_fs_id_t **successor_p,
 
   /* Create and return the new successor ID.  */
   new_id = svn_fs_base__id_create(svn_fs_base__id_node_id(id),
-                                  copy_id ? copy_id 
+                                  copy_id ? copy_id
                                   : svn_fs_base__id_copy_id(id),
                                   txn_id, pool);
 
@@ -245,7 +245,8 @@ svn_fs_bdb__put_node_revision(svn_fs_t *fs,
   skel_t *skel;
 
   /* Convert from native type into skel */
-  SVN_ERR(svn_fs_base__unparse_node_revision_skel(&skel, noderev, pool));
+  SVN_ERR(svn_fs_base__unparse_node_revision_skel(&skel, noderev, 
+                                                  bfd->format, pool));
   svn_fs_base__trail_debug(trail, "nodes", "put");
   return BDB_WRAP(fs, _("storing node revision"),
                   bfd->nodes->put(bfd->nodes, db_txn,
