@@ -44,9 +44,9 @@ svn_fs_bdb__open_reps_table(DB **reps_p,
 
   BDB_ERR(svn_fs_bdb__check_version());
   BDB_ERR(db_create(&reps, env, 0));
-  BDB_ERR(reps->open(SVN_BDB_OPEN_PARAMS(reps, NULL),
-                     "representations", 0, DB_BTREE,
-                     open_flags, 0666));
+  BDB_ERR((reps->open)(SVN_BDB_OPEN_PARAMS(reps, NULL),
+                       "representations", 0, DB_BTREE,
+                       open_flags, 0666));
 
   /* Create the `next-key' table entry.  */
   if (create)
@@ -179,8 +179,8 @@ svn_fs_bdb__write_new_rep(const char **key,
 
 
 svn_error_t *
-svn_fs_bdb__delete_rep(svn_fs_t *fs, 
-                       const char *key, 
+svn_fs_bdb__delete_rep(svn_fs_t *fs,
+                       const char *key,
                        trail_t *trail,
                        apr_pool_t *pool)
 {
