@@ -24,6 +24,8 @@
 #include "bdb/uuids-table.h"
 #include "../libsvn_fs/fs-loader.h"
 
+#include "private/svn_fs_util.h"
+
 
 struct get_uuid_args
 {
@@ -36,7 +38,7 @@ static svn_error_t *
 txn_body_get_uuid(void *baton, trail_t *trail)
 {
   struct get_uuid_args *args = baton;
-  return svn_fs_bdb__get_uuid(trail->fs, args->idx, args->uuid, 
+  return svn_fs_bdb__get_uuid(trail->fs, args->idx, args->uuid,
                               trail, trail->pool);
 }
 
@@ -48,7 +50,7 @@ svn_fs_base__get_uuid(svn_fs_t *fs,
 {
   base_fs_data_t *bfd = fs->fsap_data;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
 
   /* Check for a cached UUID first.  Failing that, we hit the
      database. */
@@ -83,7 +85,7 @@ static svn_error_t *
 txn_body_set_uuid(void *baton, trail_t *trail)
 {
   struct set_uuid_args *args = baton;
-  return svn_fs_bdb__set_uuid(trail->fs, args->idx, args->uuid, 
+  return svn_fs_bdb__set_uuid(trail->fs, args->idx, args->uuid,
                               trail, trail->pool);
 }
 
@@ -96,7 +98,10 @@ svn_fs_base__set_uuid(svn_fs_t *fs,
   struct set_uuid_args args;
   base_fs_data_t *bfd = fs->fsap_data;
 
-  SVN_ERR(svn_fs_base__check_fs(fs));
+  SVN_ERR(svn_fs__check_fs(fs, TRUE));
+
+  if (! uuid)
+    uuid = svn_uuid_generate(pool);
 
   args.idx = 1;
   args.uuid = uuid;
