@@ -2,7 +2,7 @@
  * property.c : property routines for ra_serf
  *
  * ====================================================================
- * Copyright (c) 2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2006-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -22,6 +22,10 @@
 
 #include "svn_path.h"
 #include "svn_base64.h"
+#include "svn_xml.h"
+
+#include "private/svn_dav_protocol.h"
+#include "svn_private_config.h"
 
 #include "ra_serf.h"
 
@@ -96,6 +100,9 @@ struct svn_ra_serf__propfind_context_t {
   /* Are we done issuing the PROPFIND? */
   svn_boolean_t done;
 
+  /* Context from XML stream */
+  svn_ra_serf__xml_parser_t *parser_ctx;
+
   /* If not-NULL, add us to this list when we're done. */
   svn_ra_serf__list_t **done_list;
 
@@ -263,7 +270,7 @@ start_propfind(svn_ra_serf__xml_parser_t *parser,
       info->ns = name.namespace;
       info->name = apr_pstrdup(info->pool, name.name);
       info->encoding = apr_pstrdup(info->pool,
-                                   svn_ra_serf__find_attr(attrs, "V:encoding"));
+                                   svn_xml_get_attr_value("V:encoding", attrs));
     }
 
   return SVN_NO_ERROR;
@@ -313,7 +320,7 @@ end_propfind(svn_ra_serf__xml_parser_t *parser,
               info->val_len = 0;
             }
         }
-   
+
       if (parser->state->prev->current_state == RESPONSE &&
           strcmp(name.name, "href") == 0)
         {
@@ -342,7 +349,10 @@ end_propfind(svn_ra_serf__xml_parser_t *parser,
             }
           else
             {
-              abort();
+              return svn_error_createf(SVN_ERR_RA_DAV_MALFORMED_DATA,
+                                       NULL,
+                                       _("Got unrecognized encoding '%s'"),
+                                       info->encoding);
             }
         }
 
@@ -390,6 +400,8 @@ cdata_propfind(svn_ra_serf__xml_parser_t *parser,
   prop_state_e state;
   prop_info_t *info;
 
+  UNUSED_CTX(ctx);
+
   state = parser->state->current_state;
   info = parser->state->private;
 
@@ -413,7 +425,7 @@ setup_propfind(serf_request_t *request,
                apr_pool_t *pool)
 {
   svn_ra_serf__propfind_context_t *ctx = setup_baton;
-  svn_ra_serf__xml_parser_t *parser_ctx;
+  svn_ra_serf__xml_parser_t *parser_ctx = ctx->parser_ctx;
 
   *req_bkt =
       svn_ra_serf__bucket_propfind_create(ctx->conn, ctx->path, ctx->label,
@@ -433,8 +445,6 @@ setup_propfind(serf_request_t *request,
         }
     }
 
-  parser_ctx = apr_pcalloc(pool, sizeof(*parser_ctx));
-
   parser_ctx->pool = pool;
   parser_ctx->user_data = ctx;
   parser_ctx->start = start_propfind;
@@ -490,9 +500,10 @@ check_cache(apr_hash_t *ret_props,
  * serf context for the properties listed in LOOKUP_PROPS at URL for
  * DEPTH ("0","1","infinity").
  *
- * This function will not block waiting for the response.  Instead, the
- * caller is expected to call context_run and wait for the PROP_CTX->done
- * flag to be set.
+ * This function will not block waiting for the response.  If the
+ * request can be satisfied from a local cache, set PROP_CTX to NULL
+ * as a signal to callers of that fact.  Otherwise, callers are
+ * expected to call svn_ra_serf__wait_for_props().
  */
 svn_error_t *
 svn_ra_serf__deliver_props(svn_ra_serf__propfind_context_t **prop_ctx,
@@ -519,7 +530,7 @@ svn_ra_serf__deliver_props(svn_ra_serf__propfind_context_t **prop_ctx,
 
           cache_satisfy = check_cache(ret_props, sess, path, rev, find_props,
                                       pool);
-          
+
           if (cache_satisfy)
             {
               *prop_ctx = NULL;
@@ -560,6 +571,9 @@ svn_ra_serf__deliver_props(svn_ra_serf__propfind_context_t **prop_ctx,
 
       new_prop_ctx->handler = handler;
 
+      new_prop_ctx->parser_ctx = apr_pcalloc(pool,
+                                             sizeof(*new_prop_ctx->parser_ctx));
+
       *prop_ctx = new_prop_ctx;
     }
 
@@ -590,7 +604,15 @@ svn_ra_serf__wait_for_props(svn_ra_serf__propfind_context_t *prop_ctx,
                             svn_ra_serf__session_t *sess,
                             apr_pool_t *pool)
 {
-  return svn_ra_serf__context_run_wait(&prop_ctx->done, sess, pool);
+  svn_error_t *err;
+
+  err = svn_ra_serf__context_run_wait(&prop_ctx->done, sess, pool);
+  if (prop_ctx->parser_ctx->error)
+    {
+      svn_error_clear(err);
+      SVN_ERR(prop_ctx->parser_ctx->error);
+    }
+  return err;
 }
 
 /*
@@ -618,6 +640,53 @@ svn_ra_serf__retrieve_props(apr_hash_t *prop_vals,
   return SVN_NO_ERROR;
 }
 
+svn_error_t *
+svn_ra_serf__search_for_base_props(apr_hash_t *props,
+                                   const char **remaining_path,
+                                   const char **missing_path,
+                                   svn_ra_serf__session_t *session,
+                                   svn_ra_serf__connection_t *conn,
+                                   const char *url,
+                                   apr_pool_t *pool)
+{
+  const char *path = url, *present_path = "";
+  const char *vcc_url;
+
+  do
+    {
+      SVN_ERR(svn_ra_serf__retrieve_props(props, session, conn,
+                                          path, SVN_INVALID_REVNUM,
+                                          "0", base_props, pool));
+      vcc_url =
+          svn_ra_serf__get_ver_prop(props, path,
+                                    SVN_INVALID_REVNUM,
+                                    "DAV:",
+                                    "version-controlled-configuration");
+      if (vcc_url)
+        break;
+
+      /* This happens when the file is missing in HEAD. */
+
+      /* Okay, strip off. */
+      present_path = svn_path_join(svn_path_basename(path, pool),
+                                   present_path, pool);
+      path = svn_path_dirname(path, pool);
+    }
+  while (!svn_path_is_empty(path));
+
+  /* Error out if entire URL was bogus (not a single part of it exists
+     in the repository!)  */
+  if (svn_path_is_empty(path))
+    return svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
+                             _("No part of path '%s' was found in "
+                               "repository HEAD"), url);
+
+  *missing_path = present_path;
+  *remaining_path = path;
+
+  return SVN_NO_ERROR;
+}
+
 void
 svn_ra_serf__walk_all_props(apr_hash_t *props,
                             const char *name,
@@ -740,6 +809,7 @@ set_bare_props(svn_ra_serf__prop_set_t setprop, void *baton,
 
   return setprop(baton, prop_name, val, pool);
 }
+
 svn_error_t *
 svn_ra_serf__set_baton_props(svn_ra_serf__prop_set_t setprop, void *baton,
                              const char *ns, apr_ssize_t ns_len,
@@ -757,9 +827,9 @@ svn_ra_serf__set_baton_props(svn_ra_serf__prop_set_t setprop, void *baton,
     prop_name = apr_pstrcat(pool, SVN_PROP_PREFIX, name, NULL);
   else if (strcmp(ns, "") == 0)
     prop_name = name;
-  else if (strcmp(name, "version-name") == 0)
+  else if (strcmp(name, SVN_DAV__VERSION_NAME) == 0)
     prop_name = SVN_PROP_ENTRY_COMMITTED_REV;
-  else if (strcmp(name, "creationdate") == 0)
+  else if (strcmp(name, SVN_DAV__CREATIONDATE) == 0)
     prop_name = SVN_PROP_ENTRY_COMMITTED_DATE;
   else if (strcmp(name, "creator-displayname") == 0)
     prop_name = SVN_PROP_ENTRY_LAST_AUTHOR;
@@ -769,10 +839,17 @@ svn_ra_serf__set_baton_props(svn_ra_serf__prop_set_t setprop, void *baton,
     prop_name = SVN_PROP_ENTRY_LOCK_TOKEN;
   else if (strcmp(name, "checked-in") == 0)
     prop_name = SVN_RA_SERF__WC_CHECKED_IN_URL;
+  else if (strcmp(ns, "DAV:") == 0 ||
+           strcmp(ns, SVN_DAV_PROP_NS_DAV) == 0)
+    {
+      /* Here DAV: properties not yet converted to svn: properties should be 
+         ignored. */
+      return SVN_NO_ERROR;
+    }
   else
     {
-      /* do nothing for now? */
-      return SVN_NO_ERROR;
+      /* An unknown namespace, must be a custom property. */
+      prop_name = apr_pstrcat(pool, ns, name, NULL);
     }
 
   return setprop(baton, prop_name, val, pool);
@@ -812,3 +889,63 @@ svn_ra_serf__set_bare_props(void *baton,
   return set_bare_props(set_hash_props, baton,
                         ns, ns_len, name, name_len, val, pool);
 }
+
+
+svn_error_t *
+svn_ra_serf__get_baseline_info(const char **bc_url,
+                               const char **bc_relative,
+                               svn_ra_serf__session_t *session,
+                               const char *url,
+                               svn_revnum_t revision,
+                               apr_pool_t *pool)
+{
+  const char *vcc_url, *relative_url, *basecoll_url, *baseline_url;
+  apr_hash_t *props = apr_hash_make(pool);
+
+  /* No URL?  No sweat.  We'll use the session URL. */
+  if (! url)
+    url = session->repos_url.path;
+
+  SVN_ERR(svn_ra_serf__discover_root(&vcc_url, &relative_url,
+                                     session, session->conns[0], url, pool));
+
+  if (revision != SVN_INVALID_REVNUM)
+    {
+      SVN_ERR(svn_ra_serf__retrieve_props(props, session, session->conns[0],
+                                          vcc_url, revision, "0",
+                                          baseline_props, pool));
+      basecoll_url = svn_ra_serf__get_ver_prop(props, vcc_url, revision,
+                                               "DAV:", "baseline-collection");
+    }
+  else
+    {
+      SVN_ERR(svn_ra_serf__retrieve_props(props, session, session->conns[0],
+                                          vcc_url, revision, "0",
+                                          checked_in_props, pool));
+      baseline_url = svn_ra_serf__get_ver_prop(props, vcc_url, revision,
+                                               "DAV:", "checked-in");
+      if (!baseline_url)
+        {
+          return svn_error_create(SVN_ERR_RA_DAV_OPTIONS_REQ_FAILED, NULL,
+                                  _("The OPTIONS response did not include the "
+                                    "requested checked-in value"));
+        }
+
+      SVN_ERR(svn_ra_serf__retrieve_props(props, session, session->conns[0],
+                                          baseline_url, revision, "0",
+                                          baseline_props, pool));
+      basecoll_url = svn_ra_serf__get_ver_prop(props, baseline_url, revision,
+                                               "DAV:", "baseline-collection");
+    }
+
+  if (!basecoll_url)
+    {
+      return svn_error_create(SVN_ERR_RA_DAV_OPTIONS_REQ_FAILED, NULL,
+                              _("The OPTIONS response did not include the "
+                                "requested baseline-collection value"));
+    }
+  *bc_url = basecoll_url;
+  *bc_relative = relative_url;
+  return SVN_NO_ERROR;
+}
+
