@@ -50,7 +50,7 @@ create_node(const char *name,
             svn_repos_node_t *parent,
             apr_pool_t *pool)
 {
-  svn_repos_node_t *node = apr_pcalloc(pool, sizeof(svn_repos_node_t));
+  svn_repos_node_t *node = apr_pcalloc(pool, sizeof(*node));
   node->action = 'R';
   node->kind = svn_node_unknown;
   node->name = apr_pstrdup(pool, name);
@@ -60,12 +60,12 @@ create_node(const char *name,
 
 
 static svn_repos_node_t *
-create_sibling_node(svn_repos_node_t *elder, 
-                    const char *name, 
+create_sibling_node(svn_repos_node_t *elder,
+                    const char *name,
                     apr_pool_t *pool)
 {
   svn_repos_node_t *tmp_node;
-  
+
   /* No ELDER sibling?  That's just not gonna work out. */
   if (! elder)
     return NULL;
@@ -81,8 +81,8 @@ create_sibling_node(svn_repos_node_t *elder,
 
 
 static svn_repos_node_t *
-create_child_node(svn_repos_node_t *parent, 
-                  const char *name, 
+create_child_node(svn_repos_node_t *parent,
+                  const char *name,
                   apr_pool_t *pool)
 {
   /* No PARENT node?  That's just not gonna work out. */
@@ -100,7 +100,7 @@ create_child_node(svn_repos_node_t *parent,
 
 
 static svn_repos_node_t *
-find_child_by_name(svn_repos_node_t *parent, 
+find_child_by_name(svn_repos_node_t *parent,
                    const char *name)
 {
   svn_repos_node_t *tmp_node;
@@ -138,8 +138,8 @@ find_real_base_location(const char **path_p,
 {
   /* If NODE is an add-with-history, then its real base location is
      the copy source. */
-  if ((node->action == 'A') 
-      && node->copyfrom_path 
+  if ((node->action == 'A')
+      && node->copyfrom_path
       && SVN_IS_VALID_REVNUM(node->copyfrom_rev))
     {
       *path_p = node->copyfrom_path;
@@ -205,7 +205,7 @@ delete_entry(const char *path,
   svn_revnum_t base_rev;
   svn_fs_root_t *base_root;
   svn_node_kind_t kind;
-                              
+
   /* Get (or create) the change node and update it. */
   name = svn_path_basename(path, pool);
   node = find_child_by_name(d->node, name);
@@ -263,14 +263,14 @@ add_open_helper(const char *path,
   nb->parent_baton = pb;
 
   /* Create and populate the node. */
-  nb->node = create_child_node(pb->node, svn_path_basename(path, pool), 
+  nb->node = create_child_node(pb->node, svn_path_basename(path, pool),
                                eb->node_pool);
   nb->node->kind = kind;
   nb->node->action = action;
   nb->node->copyfrom_rev = copyfrom_rev;
-  nb->node->copyfrom_path = 
+  nb->node->copyfrom_path =
     copyfrom_path ? apr_pstrdup(eb->node_pool, copyfrom_path) : NULL;
-  
+
   *child_baton = nb;
   return SVN_NO_ERROR;
 }
@@ -291,7 +291,7 @@ open_root(void *edit_baton,
   d->node->kind = svn_node_dir;
   d->node->action = 'R';
   *root_baton = d;
-  
+
   return SVN_NO_ERROR;
 }
 
@@ -319,7 +319,7 @@ add_directory(const char *path,
               void **child_baton)
 {
   SVN_ERR(add_open_helper(path, 'A', svn_node_dir, parent_baton,
-                          copyfrom_path, copyfrom_revision, 
+                          copyfrom_path, copyfrom_revision,
                           pool, child_baton));
   return SVN_NO_ERROR;
 }
@@ -348,14 +348,14 @@ add_file(const char *path,
          void **file_baton)
 {
   SVN_ERR(add_open_helper(path, 'A', svn_node_file, parent_baton,
-                          copyfrom_path, copyfrom_revision, 
+                          copyfrom_path, copyfrom_revision,
                           pool, file_baton));
   return SVN_NO_ERROR;
 }
 
 
 static svn_error_t *
-apply_textdelta(void *file_baton, 
+apply_textdelta(void *file_baton,
                 const char *base_checksum,
                 apr_pool_t *pool,
                 svn_txdelta_window_handler_t *handler,
@@ -372,7 +372,7 @@ apply_textdelta(void *file_baton,
 
 static svn_error_t *
 change_node_prop(void *node_baton,
-                 const char *name, 
+                 const char *name,
                  const svn_string_t *value,
                  apr_pool_t *pool)
 {
