@@ -46,7 +46,7 @@
    trembling with a sort of momentarily stayed spring force,
    svn_repos_dir_delta was a timebomb poised for total annihilation of
    the American Midwest.
-   
+
    Subversion needed a change.
 
    Changes, in fact.  And not just in the literary segue sense.  What
@@ -82,7 +82,7 @@
    with each change item, and whose values are arrays of changes made
    to that path, again preserving the chronological ordering.
 
-   Once our hash it built, we then sort all the keys of the hash (the
+   Once our hash is built, we then sort all the keys of the hash (the
    paths) using a depth-first directory sort routine.
 
    Finally, we drive an editor, moving down our list of sorted paths,
@@ -90,11 +90,11 @@
    and closures) needed to navigate between each successive path.  For
    each path, we replay the sorted actions that occurred at that path.
 
-   We we've finished the editor drive, we should have fully replayed
-   the filesystem events that occurred in that revision or transactions
+   When we've finished the editor drive, we should have fully replayed
+   the filesystem events that occurred in that revision or transaction
    (though not necessarily in the same order in which they
    occurred). */
-   
+
 
 
 /*** Helper functions. ***/
@@ -157,6 +157,7 @@ add_subdir(svn_fs_root_t *source_root,
            const char *source_path,
            svn_repos_authz_func_t authz_read_func,
            void *authz_read_baton,
+           apr_hash_t *changed_paths,
            apr_pool_t *pool,
            void **dir_baton)
 {
@@ -168,7 +169,7 @@ add_subdir(svn_fs_root_t *source_root,
   SVN_ERR(editor->add_directory(path, parent_baton, NULL,
                                 SVN_INVALID_REVNUM, pool, dir_baton));
 
-  SVN_ERR(svn_fs_node_proplist(&props, source_root, source_path, pool));
+  SVN_ERR(svn_fs_node_proplist(&props, target_root, path, pool));
 
   for (phi = apr_hash_first(pool, props); phi; phi = apr_hash_next(phi))
     {
@@ -185,10 +186,14 @@ add_subdir(svn_fs_root_t *source_root,
                                       subpool));
     }
 
+  /* We have to get the dirents from the source path, not the target,
+     because we want nested copies from *readable* paths to be handled by
+     path_driver_cb_func, not add_subdir (in order to preserve history). */
   SVN_ERR(svn_fs_dir_entries(&dirents, source_root, source_path, pool));
 
   for (hi = apr_hash_first(pool, dirents); hi; hi = apr_hash_next(hi))
     {
+      svn_fs_path_change_t *change;
       svn_boolean_t readable = TRUE;
       svn_fs_dirent_t *dent;
       const char *new_path;
@@ -202,6 +207,19 @@ add_subdir(svn_fs_root_t *source_root,
 
       new_path = svn_path_join(path, dent->name, subpool);
 
+      /* If a file or subdirectory of the copied directory is listed as a
+         changed path (because it was modified after the copy but before the
+         commit), we remove it from the changed_paths hash so that future
+         calls to path_driver_cb_func will ignore it. */
+      change = apr_hash_get(changed_paths, new_path, APR_HASH_KEY_STRING);
+      if (change)
+        {
+          apr_hash_set(changed_paths, new_path, APR_HASH_KEY_STRING, NULL);
+          /* If it's a delete, skip this entry. */
+          if (change->change_kind == svn_fs_path_change_delete)
+            continue;
+        }
+
       if (authz_read_func)
         SVN_ERR(authz_read_func(&readable, target_root, new_path,
                                 authz_read_baton, pool));
@@ -218,7 +236,7 @@ add_subdir(svn_fs_root_t *source_root,
                              svn_path_join(source_path, dent->name,
                                            subpool),
                              authz_read_func, authz_read_baton,
-                             subpool, &new_dir_baton));
+                             changed_paths, subpool, &new_dir_baton));
 
           SVN_ERR(editor->close_directory(new_dir_baton, subpool));
         }
@@ -227,17 +245,12 @@ add_subdir(svn_fs_root_t *source_root,
           svn_txdelta_window_handler_t delta_handler;
           void *delta_handler_baton, *file_baton;
           svn_txdelta_stream_t *delta_stream;
-          const char *new_src_path;
           unsigned char digest[APR_MD5_DIGESTSIZE];
 
-          SVN_ERR(editor->add_file(svn_path_join(path, dent->name, subpool),
-                                   *dir_baton, NULL, SVN_INVALID_REVNUM,
-                                   pool, &file_baton));
+          SVN_ERR(editor->add_file(new_path, *dir_baton, NULL,
+                                   SVN_INVALID_REVNUM, pool, &file_baton));
 
-          new_src_path = svn_path_join(source_path, dent->name, subpool);
-
-          SVN_ERR(svn_fs_node_proplist(&props, source_root, new_src_path,
-                                       subpool));
+          SVN_ERR(svn_fs_node_proplist(&props, target_root, new_path, subpool));
 
           for (phi = apr_hash_first(pool, props);
                phi;
@@ -253,12 +266,12 @@ add_subdir(svn_fs_root_t *source_root,
                                                subpool));
             }
 
-          SVN_ERR(editor->apply_textdelta(file_baton, NULL, pool, 
-                                          &delta_handler, 
+          SVN_ERR(editor->apply_textdelta(file_baton, NULL, pool,
+                                          &delta_handler,
                                           &delta_handler_baton));
 
           SVN_ERR(svn_fs_get_file_delta_stream
-                  (&delta_stream, NULL, NULL, source_root, new_src_path,
+                  (&delta_stream, NULL, NULL, target_root, new_path,
                    pool));
 
           SVN_ERR(svn_txdelta_send_txstream(delta_stream,
@@ -266,9 +279,9 @@ add_subdir(svn_fs_root_t *source_root,
                                             delta_handler_baton,
                                             pool));
 
-          SVN_ERR(svn_fs_file_md5_checksum(digest, 
-                                           source_root, 
-                                           new_src_path, 
+          SVN_ERR(svn_fs_file_md5_checksum(digest,
+                                           target_root,
+                                           new_path,
                                            pool));
           SVN_ERR(editor->close_file(file_baton,
                                      svn_md5_digest_to_cstring(digest, pool),
@@ -331,6 +344,13 @@ path_driver_cb_func(void **dir_baton,
     cb->copies->nelts--;
 
   change = apr_hash_get(cb->changed_paths, path, APR_HASH_KEY_STRING);
+  if (! change)
+    {
+      /* This can only happen if the path was removed from cb->changed_paths
+         by an earlier call to add_subdir, which means the path was already
+         handled and we should simply ignore it. */
+      return SVN_NO_ERROR;
+    }
   switch (change->change_kind)
     {
     case svn_fs_path_change_add:
@@ -354,7 +374,7 @@ path_driver_cb_func(void **dir_baton,
 
   /* Handle any deletions. */
   if (do_delete)
-    SVN_ERR(editor->delete_entry(path, SVN_INVALID_REVNUM, 
+    SVN_ERR(editor->delete_entry(path, SVN_INVALID_REVNUM,
                                  parent_baton, pool));
 
   /* Fetch the node kind if it makes sense to do so. */
@@ -362,8 +382,8 @@ path_driver_cb_func(void **dir_baton,
     {
       SVN_ERR(svn_fs_check_path(&kind, root, path, pool));
       if ((kind != svn_node_dir) && (kind != svn_node_file))
-        return svn_error_createf 
-          (SVN_ERR_FS_NOT_FOUND, NULL, 
+        return svn_error_createf
+          (SVN_ERR_FS_NOT_FOUND, NULL,
            _("Filesystem path '%s' is neither a file nor a directory"), path);
     }
 
@@ -416,7 +436,7 @@ path_driver_cb_func(void **dir_baton,
               SVN_ERR(add_subdir(copyfrom_root, root, editor, edit_baton,
                                  path, parent_baton, real_copyfrom_path,
                                  cb->authz_read_func, cb->authz_read_baton,
-                                 pool, dir_baton));
+                                 cb->changed_paths, pool, dir_baton));
             }
           else
             {
@@ -477,13 +497,13 @@ path_driver_cb_func(void **dir_baton,
         {
           if (parent_baton)
             {
-              SVN_ERR(editor->open_directory(path, parent_baton, 
+              SVN_ERR(editor->open_directory(path, parent_baton,
                                              SVN_INVALID_REVNUM,
                                              pool, dir_baton));
             }
           else
             {
-              SVN_ERR(editor->open_root(edit_baton, SVN_INVALID_REVNUM, 
+              SVN_ERR(editor->open_root(edit_baton, SVN_INVALID_REVNUM,
                                         pool, dir_baton));
             }
         }
@@ -588,8 +608,8 @@ path_driver_cb_func(void **dir_baton,
               checksum = svn_md5_digest_to_cstring(digest, pool);
             }
 
-          SVN_ERR(editor->apply_textdelta(file_baton, checksum, pool, 
-                                          &delta_handler, 
+          SVN_ERR(editor->apply_textdelta(file_baton, checksum, pool,
+                                          &delta_handler,
                                           &delta_handler_baton));
           if (cb->compare_root)
             {
@@ -614,7 +634,7 @@ path_driver_cb_func(void **dir_baton,
     {
       unsigned char digest[APR_MD5_DIGESTSIZE];
       SVN_ERR(svn_fs_file_md5_checksum(digest, root, path, pool));
-      SVN_ERR(editor->close_file(file_baton, 
+      SVN_ERR(editor->close_file(file_baton,
                                  svn_md5_digest_to_cstring(digest, pool),
                                  pool));
     }
@@ -690,14 +710,21 @@ svn_repos_replay2(svn_fs_root_t *root,
               APR_ARRAY_PUSH(paths, const char *) = path;
               apr_hash_set(changed_paths, path, keylen, change);
             }
+          /* ...unless this was a change to one of the parent directories of 
+             base_path. */
+          else if (is_within_base_path(base_path, path, keylen))
+            {
+              APR_ARRAY_PUSH(paths, const char *) = path;
+              apr_hash_set(changed_paths, path, keylen, change);
+            }
         }
     }
 
   /* If we were not given a low water mark, assume that everything is there,
-     all the way back to revision 0. */ 
+     all the way back to revision 0. */
   if (! SVN_IS_VALID_REVNUM(low_water_mark))
     low_water_mark = 0;
- 
+
   /* Initialize our callback baton. */
   cb_baton.editor = editor;
   cb_baton.edit_baton = edit_baton;
@@ -708,29 +735,17 @@ svn_repos_replay2(svn_fs_root_t *root,
   cb_baton.base_path = base_path;
   cb_baton.base_path_len = base_path_len;
   cb_baton.low_water_mark = low_water_mark;
+  cb_baton.compare_root = NULL;
 
   if (send_deltas)
     {
-      svn_revnum_t revision = svn_fs_revision_root_revision(root);
-      svn_revnum_t compare_root_revision;
-      if (! SVN_IS_VALID_REVNUM(revision))
-        {
-          svn_fs_txn_t *txn;
-          const char *txn_name = svn_fs_txn_root_name(root, pool);
-          SVN_ERR(svn_fs_open_txn(&txn, svn_fs_root_fs(root),
-                                  txn_name, pool));
-          compare_root_revision = svn_fs_txn_base_revision(txn);
-        }
-      else
-        compare_root_revision = revision - 1;
-
       SVN_ERR(svn_fs_revision_root(&cb_baton.compare_root,
                                    svn_fs_root_fs(root),
-                                   compare_root_revision,
+                                   svn_fs_is_revision_root(root)
+                                     ? svn_fs_revision_root_revision(root) - 1
+                                     : svn_fs_txn_root_base_revision(root),
                                    pool));
     }
-  else
-    cb_baton.compare_root = NULL;
 
   cb_baton.copies = apr_array_make(pool, 4, sizeof(struct copy_info));
   cb_baton.pool = pool;
@@ -744,10 +759,10 @@ svn_repos_replay2(svn_fs_root_t *root,
     }
 
   /* Call the path-based editor driver. */
-  SVN_ERR(svn_delta_path_driver(editor, edit_baton, 
-                                SVN_INVALID_REVNUM, paths, 
+  SVN_ERR(svn_delta_path_driver(editor, edit_baton,
+                                SVN_INVALID_REVNUM, paths,
                                 path_driver_cb_func, &cb_baton, pool));
-  
+
   return SVN_NO_ERROR;
 }
 
