@@ -2,7 +2,7 @@
  * utf.c:  UTF-8 conversion routines
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -32,6 +32,7 @@
 #include "svn_utf.h"
 #include "utf_impl.h"
 #include "svn_private_config.h"
+#include "win32_xlate.h"
 
 
 
@@ -124,7 +125,7 @@ svn_utf_initialize(apr_pool_t *pool)
       else
         return;
 #endif
-      
+
       xlate_handle_hash = apr_hash_make(subpool);
       apr_pool_cleanup_register(subpool, NULL, xlate_cleanup,
                                 apr_pool_cleanup_null);
@@ -252,10 +253,13 @@ get_xlate_handle_node(xlate_handle_node_t **ret,
     pool = apr_hash_pool_get(xlate_handle_hash);
 
   /* Try to create a handle. */
-#ifndef AS400
-  apr_err = apr_xlate_open(&handle, topage, frompage, pool);
-#else
+#if defined( WIN32)
+  apr_err = svn_subr__win32_xlate_open((win32_xlate_t **)&handle, topage,
+                                       frompage, pool);
+#elif defined(AS400)
   apr_err = apr_xlate_open(&handle, (int)topage, (int)frompage, pool);
+#else
+  apr_err = apr_xlate_open(&handle, topage, frompage, pool);
 #endif
 
   if (APR_STATUS_IS_EINVAL(apr_err) || APR_STATUS_IS_ENOTIMPL(apr_err))
@@ -454,10 +458,17 @@ convert_to_stringbuf(xlate_handle_node_t *node,
                      svn_stringbuf_t **dest,
                      apr_pool_t *pool)
 {
-  apr_size_t buflen = src_length;
+#ifdef WIN32
+  apr_status_t apr_err;
+
+  apr_err = svn_subr__win32_xlate_to_stringbuf((win32_xlate_t *) node->handle,
+                                               src_data, src_length,
+                                               dest, pool);
+#else
+  apr_size_t buflen = src_length * 2;
   apr_status_t apr_err;
   apr_size_t srclen = src_length;
-  apr_size_t destlen = 0;
+  apr_size_t destlen = buflen;
   char *destbuf;
 
   /* Initialize *DEST to an empty stringbuf. */
@@ -469,37 +480,35 @@ convert_to_stringbuf(xlate_handle_node_t *node,
   if (src_length == 0)
     return SVN_NO_ERROR;
 
-  do 
+  do
     {
-      /* A 1:2 ratio of input characters to output characters should
-         be enough for most translations, and conveniently enough, if
-         it isn't, we'll grow the buffer size by 2 again. */
-      if (destlen == 0)
-        buflen *= 2;
+      /* A 1:2 ratio of input bytes to output bytes (as assigned above)
+         should be enough for most translations, and if it turns out not
+         to be enough, we'll grow the buffer again, sizing it based on a
+         1:3 ratio of the remainder of the string. */
 
-      /* Ensure that *DEST has sufficient storage for the translated
-         result. */
       svn_stringbuf_ensure(*dest, buflen + 1);
 
-      /* Update the destination buffer pointer to the first character
-         after already-converted output. */
-      destbuf = (*dest)->data + (*dest)->len;
-
       /* Set up state variables for xlate. */
       destlen = buflen - (*dest)->len;
 
       /* Attempt the conversion. */
       apr_err = apr_xlate_conv_buffer(node->handle,
-                                      src_data + (src_length - srclen), 
+                                      src_data + (src_length - srclen),
                                       &srclen,
-                                      destbuf, 
+                                      (*dest)->data + (*dest)->len,
                                       &destlen);
 
       /* Now, update the *DEST->len to track the amount of output data
          churned out so far from this loop. */
       (*dest)->len += ((buflen - (*dest)->len) - destlen);
+      buflen += srclen * 3; /* 3 is middle ground, 2 wasn't enough
+                               for all characters in the buffer, 4 is
+                               maximum character size (currently) */
 
-    } while (! apr_err && srclen);
+
+    } while (apr_err == APR_SUCCESS && srclen != 0);
+#endif
 
   /* If we exited the loop with an error, return the error. */
   if (apr_err)
@@ -562,7 +571,7 @@ check_non_ascii(const char *data, apr_size_t len, apr_pool_t *pool)
              time tracking down the non-ASCII data, so we want to help
              as much as possible.  And yes, we just call the unsafe
              data "non-ASCII", even though the actual constraint is
-             somewhat more complex than that. */ 
+             somewhat more complex than that. */
 
           if (data - data_start)
             {
