@@ -16,19 +16,18 @@
  * ====================================================================
  */
 
-
+#include <apr_hash.h>
 
 #include <httpd.h>
 #include <mod_dav.h>
 
-#include <apr_hash.h>
-
-#include "dav_svn.h"
-
 #include "svn_xml.h"
 #include "svn_pools.h"
 #include "svn_dav.h"
 #include "svn_base64.h"
+#include "svn_props.h"
+
+#include "dav_svn.h"
 
 
 struct dav_db {
@@ -41,8 +40,13 @@ struct dav_db {
 
   /* used for constructing repos-local names for properties */
   svn_stringbuf_t *work;
+
+  /* passed to svn_repos_ funcs that fetch revprops. */
+  svn_repos_authz_func_t authz_read_func;
+  void *authz_read_baton;
 };
 
+
 struct dav_deadprop_rollback {
   dav_prop_name name;
   svn_string_t value;
@@ -50,15 +54,18 @@ struct dav_deadprop_rollback {
 
 
 /* retrieve the "right" string to use as a repos path */
-static const char *get_repos_path (struct dav_resource_private *info)
+static const char *
+get_repos_path(struct dav_resource_private *info)
 {
   return info->repos_path;
 }
 
 
 /* construct the repos-local name for the given DAV property name */
-static void get_repos_propname(dav_db *db, const dav_prop_name *name,
-                               const char **repos_propname)
+static void
+get_repos_propname(dav_db *db,
+                   const dav_prop_name *name,
+                   const char **repos_propname)
 {
   if (strcmp(name->ns, SVN_DAV_PROP_NS_SVN) == 0)
     {
@@ -78,8 +85,9 @@ static void get_repos_propname(dav_db *db, const dav_prop_name *name,
     }
 }
 
-static dav_error *get_value(dav_db *db, const dav_prop_name *name,
-                            svn_string_t **pvalue)
+
+static dav_error *
+get_value(dav_db *db, const dav_prop_name *name, svn_string_t **pvalue)
 {
   const char *propname;
   svn_error_t *serr;
@@ -102,23 +110,26 @@ static dav_error *get_value(dav_db *db, const dav_prop_name *name,
       serr = svn_fs_txn_prop(pvalue, db->resource->info->root.txn,
                              propname, db->p);
     else
-      serr = svn_fs_revision_prop(pvalue, db->resource->info->repos->fs,
-                                  db->resource->info->root.rev,
-                                  propname, db->p);
+      serr = svn_repos_fs_revision_prop(pvalue,
+                                        db->resource->info-> repos->repos,
+                                        db->resource->info->root.rev, propname,
+                                        db->authz_read_func,
+                                        db->authz_read_baton, db->p);
   else
     serr = svn_fs_node_prop(pvalue, db->resource->info->root.root,
                             get_repos_path(db->resource->info),
                             propname, db->p);
   if (serr != NULL)
-    return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                               "could not fetch a property",
-                               db->resource->pool);
+    return dav_svn__convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                "could not fetch a property",
+                                db->resource->pool);
 
   return NULL;
 }
 
-static dav_error *save_value(dav_db *db, const dav_prop_name *name,
-                             const svn_string_t *value)
+
+static dav_error *
+save_value(dav_db *db, const dav_prop_name *name, const svn_string_t *value)
 {
   const char *propname;
   svn_error_t *serr;
@@ -144,22 +155,36 @@ static dav_error *save_value(dav_db *db, const dav_prop_name *name,
       serr = svn_repos_fs_change_txn_prop(db->resource->info->root.txn,
                                           propname, value, db->resource->pool);
     else
-      /* ### VIOLATING deltaV: you can't proppatch a baseline, it's
-         not a working resource!  But this is how we currently
-         (hackily) allow the svn client to change unversioned rev
-         props.  See issue #916. */
-      serr = svn_repos_fs_change_rev_prop(db->resource->info->repos->repos,
-                                          db->resource->info->root.rev,
-                                          db->resource->info->repos->username,
-                                          propname, value, db->resource->pool);
+      {
+        /* ### VIOLATING deltaV: you can't proppatch a baseline, it's
+           not a working resource!  But this is how we currently
+           (hackily) allow the svn client to change unversioned rev
+           props.  See issue #916. */
+        serr = svn_repos_fs_change_rev_prop3
+          (db->resource->info->repos->repos,
+           db->resource->info->root.rev,
+           db->resource->info->repos->username,
+           propname, value, TRUE, TRUE,
+           db->authz_read_func,
+           db->authz_read_baton,
+           db->resource->pool);
+
+        /* Tell the logging subsystem about the revprop change. */
+        dav_svn__operational_log(db->resource->info,
+                                 apr_psprintf(db->resource->pool,
+                                   "change-rev-prop r%ld %s",
+                                   db->resource->info->root.rev,
+                                   svn_path_uri_encode(propname,
+                                                       db->resource->pool)));
+      }
   else
     serr = svn_repos_fs_change_node_prop(db->resource->info->root.root,
                                          get_repos_path(db->resource->info),
                                          propname, value, db->resource->pool);
   if (serr != NULL)
-    return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                               NULL,
-                               db->resource->pool);
+    return dav_svn__convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                NULL,
+                                db->resource->pool);
 
   /* a change to the props was made; make sure our cached copy is gone */
   db->props = NULL;
@@ -167,10 +192,15 @@ static dav_error *save_value(dav_db *db, const dav_prop_name *name,
   return NULL;
 }
 
-static dav_error *dav_svn_db_open(apr_pool_t *p, const dav_resource *resource,
-                                  int ro, dav_db **pdb)
+
+static dav_error *
+db_open(apr_pool_t *p,
+        const dav_resource *resource,
+        int ro,
+        dav_db **pdb)
 {
   dav_db *db;
+  dav_svn__authz_read_baton *arb;
 
   /* Some resource types do not have deadprop databases. Specifically:
      REGULAR, VERSION, and WORKING resources have them. (SVN does not
@@ -205,6 +235,13 @@ static dav_error *dav_svn_db_open(apr_pool_t *p, const dav_resource *resource,
   /* ### temp hack */
   db->work = svn_stringbuf_ncreate("", 0, db->p);
 
+  /* make our path-based authz callback available to svn_repos_* funcs. */
+  arb = apr_pcalloc(p, sizeof(*arb));
+  arb->r = resource->info->r;
+  arb->repos = resource->info->repos;
+  db->authz_read_baton = arb;
+  db->authz_read_func = dav_svn__authz_read_func(arb);
+
   /* ### use RO and node's mutable status to look for an error? */
 
   *pdb = db;
@@ -212,12 +249,16 @@ static dav_error *dav_svn_db_open(apr_pool_t *p, const dav_resource *resource,
   return NULL;
 }
 
-static void dav_svn_db_close(dav_db *db)
+
+static void
+db_close(dav_db *db)
 {
   svn_pool_destroy(db->p);
 }
 
-static dav_error *dav_svn_db_define_namespaces(dav_db *db, dav_xmlns_info *xi)
+
+static dav_error *
+db_define_namespaces(dav_db *db, dav_xmlns_info *xi)
 {
   dav_xmlns_add(xi, "S", SVN_DAV_PROP_NS_SVN);
   dav_xmlns_add(xi, "C", SVN_DAV_PROP_NS_CUSTOM);
@@ -228,10 +269,12 @@ static dav_error *dav_svn_db_define_namespaces(dav_db *db, dav_xmlns_info *xi)
   return NULL;
 }
 
-static dav_error *dav_svn_db_output_value(dav_db *db, 
-                                          const dav_prop_name *name,
-                                          dav_xmlns_info *xi,
-                                          apr_text_header *phdr, int *found)
+static dav_error *
+db_output_value(dav_db *db,
+                const dav_prop_name *name,
+                dav_xmlns_info *xi,
+                apr_text_header *phdr,
+                int *found)
 {
   const char *prefix;
   const char *s;
@@ -268,12 +311,13 @@ static dav_error *dav_svn_db_output_value(dav_db *db,
          across the wire. */
       if (! svn_xml_is_xml_safe(propval->data, propval->len))
         {
-          propval = (svn_string_t *)svn_base64_encode_string(propval, pool);
-          xml_safe = propval->data;
+          const svn_string_t *enc_propval
+            = svn_base64_encode_string(propval, pool);
+          xml_safe = enc_propval->data;
           encoding = apr_pstrcat(pool, " V:encoding=\"base64\"", NULL);
         }
       else
-        {    
+        {
           svn_stringbuf_t *xmlval = NULL;
           svn_xml_escape_cdata_string(&xmlval, propval, pool);
           xml_safe = xmlval->data;
@@ -285,7 +329,7 @@ static dav_error *dav_svn_db_output_value(dav_db *db,
       /* the value is in our pool which means it has the right lifetime. */
       /* ### at least, per the current mod_dav architecture/API */
       apr_text_append(pool, phdr, xml_safe);
-      
+
       s = apr_psprintf(pool, "</%s%s>" DEBUG_CR, prefix, name->name);
       apr_text_append(pool, phdr, s);
     }
@@ -293,21 +337,24 @@ static dav_error *dav_svn_db_output_value(dav_db *db,
   return NULL;
 }
 
-static dav_error *dav_svn_db_map_namespaces(
-    dav_db *db,
-    const apr_array_header_t *namespaces,
-    dav_namespace_map **mapping)
+
+static dav_error *
+db_map_namespaces(dav_db *db,
+                  const apr_array_header_t *namespaces,
+                  dav_namespace_map **mapping)
 {
   /* we don't need a namespace mapping right now. nothing to do */
-
   return NULL;
 }
 
-static dav_error *dav_svn_db_store(dav_db *db, const dav_prop_name *name,
-                                   const apr_xml_elem *elem,
-                                   dav_namespace_map *mapping)
+
+static dav_error *
+db_store(dav_db *db,
+         const dav_prop_name *name,
+         const apr_xml_elem *elem,
+         dav_namespace_map *mapping)
 {
-  svn_string_t *propval = apr_pcalloc(db->p, sizeof(*propval));
+  const svn_string_t *propval;
   apr_pool_t *pool = db->p;
   apr_xml_attr *attr = elem->attr;
 
@@ -317,22 +364,22 @@ static dav_error *dav_svn_db_store(dav_db *db, const dav_prop_name *name,
      dav_xml_get_cdata() will figure it all out for us, but (normally) it
      should be awfully fast and not need to copy any data. */
 
-  propval->data = dav_xml_get_cdata(elem, pool, 0 /* strip_white */);
-  propval->len = strlen(propval->data);
-  
+  propval = svn_string_create
+    (dav_xml_get_cdata(elem, pool, 0 /* strip_white */), pool);
+
   /* Check for special encodings of the property value. */
   while (attr)
     {
-      if (strcmp (attr->name, "encoding") == 0) /* ### namespace check? */
+      if (strcmp(attr->name, "encoding") == 0) /* ### namespace check? */
         {
           const char *enc_type = attr->value;
 
           /* Handle known encodings here. */
-          if (enc_type && (strcmp (enc_type, "base64") == 0))
-            propval = (svn_string_t *)svn_base64_decode_string(propval, pool);
+          if (enc_type && (strcmp(enc_type, "base64") == 0))
+            propval = svn_base64_decode_string(propval, pool);
           else
-            return dav_new_error (pool, HTTP_INTERNAL_SERVER_ERROR, 0,
-                                  "Unknown property encoding");
+            return dav_new_error(pool, HTTP_INTERNAL_SERVER_ERROR, 0,
+                                 "Unknown property encoding");
           break;
         }
       /* Next attribute, please. */
@@ -342,7 +389,9 @@ static dav_error *dav_svn_db_store(dav_db *db, const dav_prop_name *name,
   return save_value(db, name, propval);
 }
 
-static dav_error *dav_svn_db_remove(dav_db *db, const dav_prop_name *name)
+
+static dav_error *
+db_remove(dav_db *db, const dav_prop_name *name)
 {
   svn_error_t *serr;
   const char *propname;
@@ -364,18 +413,21 @@ static dav_error *dav_svn_db_remove(dav_db *db, const dav_prop_name *name)
          not a working resource!  But this is how we currently
          (hackily) allow the svn client to change unversioned rev
          props.  See issue #916. */
-      serr = svn_repos_fs_change_rev_prop(db->resource->info->repos->repos,
-                                          db->resource->info->root.rev,
-                                          db->resource->info->repos->username,
-                                          propname, NULL, db->resource->pool);
+      serr = svn_repos_fs_change_rev_prop3(db->resource->info->repos->repos,
+                                           db->resource->info->root.rev,
+                                           db->resource->info->repos->username,
+                                           propname, NULL, TRUE, TRUE,
+                                           db->authz_read_func,
+                                           db->authz_read_baton,
+                                           db->resource->pool);
   else
     serr = svn_repos_fs_change_node_prop(db->resource->info->root.root,
                                          get_repos_path(db->resource->info),
                                          propname, NULL, db->resource->pool);
   if (serr != NULL)
-    return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                               "could not remove a property",
-                               db->resource->pool);
+    return dav_svn__convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                "could not remove a property",
+                                db->resource->pool);
 
   /* a change to the props was made; make sure our cached copy is gone */
   db->props = NULL;
@@ -383,11 +435,14 @@ static dav_error *dav_svn_db_remove(dav_db *db, const dav_prop_name *name)
   return NULL;
 }
 
-static int dav_svn_db_exists(dav_db *db, const dav_prop_name *name)
+
+static int
+db_exists(dav_db *db, const dav_prop_name *name)
 {
   const char *propname;
   svn_string_t *propval;
   svn_error_t *serr;
+  int retval;
 
   /* get the repos-local name */
   get_repos_propname(db, name, &propname);
@@ -402,9 +457,12 @@ static int dav_svn_db_exists(dav_db *db, const dav_prop_name *name)
       serr = svn_fs_txn_prop(&propval, db->resource->info->root.txn,
                              propname, db->p);
     else
-      serr = svn_fs_revision_prop(&propval, db->resource->info->repos->fs,
-                                  db->resource->info->root.rev,
-                                  propname, db->p);
+      serr = svn_repos_fs_revision_prop(&propval,
+                                        db->resource->info->repos->repos,
+                                        db->resource->info->root.rev,
+                                        propname,
+                                        db->authz_read_func,
+                                        db->authz_read_baton, db->p);
   else
     serr = svn_fs_node_prop(&propval, db->resource->info->root.root,
                             get_repos_path(db->resource->info),
@@ -412,7 +470,9 @@ static int dav_svn_db_exists(dav_db *db, const dav_prop_name *name)
 
   /* ### try and dispose of the value? */
 
-  return serr == NULL && propval != NULL;
+  retval = (serr == NULL && propval != NULL);
+  svn_error_clear(serr);
+  return retval;
 }
 
 static void get_name(dav_db *db, dav_prop_name *pname)
@@ -442,8 +502,13 @@ static void get_name(dav_db *db, dav_prop_name *pname)
     }
 }
 
-static dav_error *dav_svn_db_first_name(dav_db *db, dav_prop_name *pname)
+
+static dav_error *
+db_first_name(dav_db *db, dav_prop_name *pname)
 {
+  /* for operational logging */
+  char *action = NULL;
+
   /* if we don't have a copy of the properties, then get one */
   if (db->props == NULL)
     {
@@ -453,27 +518,46 @@ static dav_error *dav_svn_db_first_name(dav_db *db, dav_prop_name *pname)
       if (db->resource->baselined)
         {
           if (db->resource->type == DAV_RESOURCE_TYPE_WORKING)
-            serr = svn_fs_txn_proplist(&db->props, 
+            serr = svn_fs_txn_proplist(&db->props,
                                        db->resource->info->root.txn,
                                        db->p);
           else
-            serr = svn_fs_revision_proplist(&db->props,
-                                            db->resource->info->repos->fs,
-                                            db->resource->info->root.rev, 
-                                            db->p);
+            {
+              action = apr_psprintf(db->resource->pool, "rev-proplist r%ld",
+                                    db->resource->info->root.rev);
+              serr = svn_repos_fs_revision_proplist
+                (&db->props,
+                 db->resource->info->repos->repos,
+                 db->resource->info->root.rev,
+                 db->authz_read_func,
+                 db->authz_read_baton,
+                 db->p);
+            }
         }
       else
         {
-          serr = svn_fs_node_proplist(&db->props, 
+          svn_node_kind_t kind;
+          serr = svn_fs_node_proplist(&db->props,
                                       db->resource->info->root.root,
-                                      get_repos_path(db->resource->info), 
+                                      get_repos_path(db->resource->info),
                                       db->p);
+          if (! serr)
+            serr = svn_fs_check_path(&kind, db->resource->info->root.root,
+                                     get_repos_path(db->resource->info),
+                                     db->p);
+
+          if (! serr)
+            action = apr_psprintf(db->resource->pool, "get-%s %s r%ld props",
+                                  (kind == svn_node_dir ? "dir" : "file"),
+                                  svn_path_uri_encode(db->resource->info->repos_path,
+                                                      db->resource->pool),
+                                  db->resource->info->root.rev);
         }
       if (serr != NULL)
-        return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                                   "could not begin sequencing through "
-                                   "properties",
-                                   db->resource->pool);
+        return dav_svn__convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                    "could not begin sequencing through "
+                                    "properties",
+                                    db->resource->pool);
     }
 
   /* begin the iteration over the hash */
@@ -482,10 +566,16 @@ static dav_error *dav_svn_db_first_name(dav_db *db, dav_prop_name *pname)
   /* fetch the first key */
   get_name(db, pname);
 
+  /* If we have a high-level action to log, do so. */
+  if (action != NULL)
+    dav_svn__operational_log(db->resource->info, action);
+
   return NULL;
 }
 
-static dav_error *dav_svn_db_next_name(dav_db *db, dav_prop_name *pname)
+
+static dav_error *
+db_next_name(dav_db *db, dav_prop_name *pname)
 {
   /* skip to the next hash entry */
   if (db->hi != NULL)
@@ -497,9 +587,11 @@ static dav_error *dav_svn_db_next_name(dav_db *db, dav_prop_name *pname)
   return NULL;
 }
 
-static dav_error *dav_svn_db_get_rollback(dav_db *db, 
-                                          const dav_prop_name *name,
-                                          dav_deadprop_rollback **prollback)
+
+static dav_error *
+db_get_rollback(dav_db *db,
+                const dav_prop_name *name,
+                dav_deadprop_rollback **prollback)
 {
   dav_error *err;
   dav_deadprop_rollback *ddp;
@@ -517,29 +609,30 @@ static dav_error *dav_svn_db_get_rollback(dav_db *db,
   return NULL;
 }
 
-static dav_error *dav_svn_db_apply_rollback(dav_db *db,
-                                            dav_deadprop_rollback *rollback)
+
+static dav_error *
+db_apply_rollback(dav_db *db, dav_deadprop_rollback *rollback)
 {
   if (rollback->value.data == NULL)
     {
-      return dav_svn_db_remove(db, &rollback->name);
+      return db_remove(db, &rollback->name);
     }
-  
+
   return save_value(db, &rollback->name, &rollback->value);
 }
 
 
-const dav_hooks_propdb dav_svn_hooks_propdb = {
-  dav_svn_db_open,
-  dav_svn_db_close,
-  dav_svn_db_define_namespaces,
-  dav_svn_db_output_value,
-  dav_svn_db_map_namespaces,
-  dav_svn_db_store,
-  dav_svn_db_remove,
-  dav_svn_db_exists,
-  dav_svn_db_first_name,
-  dav_svn_db_next_name,
-  dav_svn_db_get_rollback,
-  dav_svn_db_apply_rollback,
+const dav_hooks_propdb dav_svn__hooks_propdb = {
+  db_open,
+  db_close,
+  db_define_namespaces,
+  db_output_value,
+  db_map_namespaces,
+  db_store,
+  db_remove,
+  db_exists,
+  db_first_name,
+  db_next_name,
+  db_get_rollback,
+  db_apply_rollback,
 };
