@@ -27,9 +27,9 @@
 #include "svn_pools.h"
 #include "svn_fs.h"
 #include "svn_props.h"
+#include "svn_xml.h"
 
 #include "dav_svn.h"
-#include "svn_xml.h"
 
 
 /* #################################################################
@@ -60,33 +60,34 @@
 */
 
 /* send a response to the client for this baton */
-static svn_error_t *send_response(const dav_svn_repos *repos,
-                                  svn_fs_root_t *root,
-                                  const char *path,
-                                  svn_boolean_t is_dir,
-                                  ap_filter_t *output,
-                                  apr_bucket_brigade *bb,
-                                  apr_pool_t *pool)
+static svn_error_t *
+send_response(const dav_svn_repos *repos,
+              svn_fs_root_t *root,
+              const char *path,
+              svn_boolean_t is_dir,
+              ap_filter_t *output,
+              apr_bucket_brigade *bb,
+              apr_pool_t *pool)
 {
   const char *href;
   const char *vsn_url;
   apr_status_t status;
   svn_revnum_t rev_to_use;
 
-  href = dav_svn_build_uri(repos, DAV_SVN_BUILD_URI_PUBLIC,
-                           SVN_IGNORED_REVNUM, path, 0 /* add_href */, pool);
-  rev_to_use = dav_svn_get_safe_cr(root, path, pool);
-  vsn_url = dav_svn_build_uri(repos, DAV_SVN_BUILD_URI_VERSION,
-                              rev_to_use, path, 0 /* add_href */, pool);
+  href = dav_svn__build_uri(repos, DAV_SVN__BUILD_URI_PUBLIC,
+                            SVN_IGNORED_REVNUM, path, 0 /* add_href */, pool);
+  rev_to_use = dav_svn__get_safe_cr(root, path, pool);
+  vsn_url = dav_svn__build_uri(repos, DAV_SVN__BUILD_URI_VERSION,
+                               rev_to_use, path, 0 /* add_href */, pool);
   status = ap_fputstrs(output, bb,
                        "<D:response>" DEBUG_CR
-                       "<D:href>", 
+                       "<D:href>",
                        apr_xml_quote_string(pool, href, 1),
                        "</D:href>" DEBUG_CR
                        "<D:propstat><D:prop>" DEBUG_CR,
-                       is_dir 
+                       is_dir
                          ? "<D:resourcetype><D:collection/></D:resourcetype>"
-                         : "<D:resourcetype/>", 
+                         : "<D:resourcetype/>",
                        DEBUG_CR,
                        "<D:checked-in><D:href>",
                        apr_xml_quote_string(pool, vsn_url, 1),
@@ -103,12 +104,13 @@ static svn_error_t *send_response(const dav_svn_repos *repos,
 }
 
 
-static svn_error_t *do_resources(const dav_svn_repos *repos,
-                                 svn_fs_root_t *root, 
-                                 svn_revnum_t revision, 
-                                 ap_filter_t *output,
-                                 apr_bucket_brigade *bb,
-                                 apr_pool_t *pool)
+static svn_error_t *
+do_resources(const dav_svn_repos *repos,
+             svn_fs_root_t *root,
+             svn_revnum_t revision,
+             ap_filter_t *output,
+             apr_bucket_brigade *bb,
+             apr_pool_t *pool)
 {
   apr_hash_t *changes;
   apr_hash_t *sent = apr_hash_make(pool);
@@ -166,7 +168,7 @@ static svn_error_t *do_resources(const dav_svn_repos *repos,
               svn_node_kind_t kind;
               SVN_ERR(svn_fs_check_path(&kind, root, path, subpool));
               SVN_ERR(send_response(repos, root, path,
-                                    kind == svn_node_dir ? TRUE : FALSE, 
+                                    kind == svn_node_dir ? TRUE : FALSE,
                                     output, bb, subpool));
               apr_hash_set(sent, path, APR_HASH_KEY_STRING, (void *)1);
             }
@@ -180,7 +182,7 @@ static svn_error_t *do_resources(const dav_svn_repos *repos,
           const char *parent = svn_path_dirname(path, pool);
           if (! apr_hash_get(sent, parent, APR_HASH_KEY_STRING))
             {
-              SVN_ERR(send_response(repos, root, parent, 
+              SVN_ERR(send_response(repos, root, parent,
                                     TRUE, output, bb, subpool));
               apr_hash_set(sent, parent, APR_HASH_KEY_STRING, (void *)1);
             }
@@ -198,13 +200,14 @@ static svn_error_t *do_resources(const dav_svn_repos *repos,
    PUBLIC FUNCTIONS
 */
 
-dav_error * dav_svn__merge_response(ap_filter_t *output,
-                                    const dav_svn_repos *repos,
-                                    svn_revnum_t new_rev,
-                                    char *post_commit_err,
-                                    apr_xml_elem *prop_elem,
-                                    svn_boolean_t disable_merge_response,
-                                    apr_pool_t *pool)
+dav_error *
+dav_svn__merge_response(ap_filter_t *output,
+                        const dav_svn_repos *repos,
+                        svn_revnum_t new_rev,
+                        char *post_commit_err,
+                        apr_xml_elem *prop_elem,
+                        svn_boolean_t disable_merge_response,
+                        apr_pool_t *pool)
 {
   apr_bucket_brigade *bb;
   svn_fs_root_t *root;
@@ -218,19 +221,19 @@ dav_error * dav_svn__merge_response(ap_filter_t *output,
   serr = svn_fs_revision_root(&root, repos->fs, new_rev, pool);
   if (serr != NULL)
     {
-      return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                                 "Could not open the FS root for the "
-                                 "revision just committed.",
-                                 repos->pool);
+      return dav_svn__convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                  "Could not open the FS root for the "
+                                  "revision just committed.",
+                                  repos->pool);
     }
 
   bb = apr_brigade_create(pool, output->c->bucket_alloc);
 
   /* prep some strings */
-  
+
   /* the HREF for the baseline is actually the VCC */
-  vcc = dav_svn_build_uri(repos, DAV_SVN_BUILD_URI_VCC, SVN_IGNORED_REVNUM,
-                          NULL, 0 /* add_href */, pool);
+  vcc = dav_svn__build_uri(repos, DAV_SVN__BUILD_URI_VCC, SVN_IGNORED_REVNUM,
+                           NULL, 0 /* add_href */, pool);
 
   /* the version-name of the baseline is the revision number */
   rev = apr_psprintf(pool, "%ld", new_rev);
@@ -258,17 +261,17 @@ dav_error * dav_svn__merge_response(ap_filter_t *output,
                               SVN_PROP_REVISION_DATE, pool);
   if (serr != NULL)
     {
-      return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                                 "Could not get date of newest revision",
-                                 repos->pool); 
+      return dav_svn__convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                  "Could not get date of newest revision",
+                                  repos->pool);
     }
   serr = svn_fs_revision_prop(&creator_displayname, repos->fs, new_rev,
                               SVN_PROP_REVISION_AUTHOR, pool);
   if (serr != NULL)
     {
-      return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                                 "Could not get author of newest revision",
-                                 repos->pool); 
+      return dav_svn__convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                  "Could not get author of newest revision",
+                                  repos->pool);
     }
 
 
@@ -281,7 +284,7 @@ dav_error * dav_svn__merge_response(ap_filter_t *output,
 
                      /* generate a response for the new baseline */
                      "<D:response>" DEBUG_CR
-                     "<D:href>", 
+                     "<D:href>",
                      apr_xml_quote_string(pool, vcc, 1),
                      "</D:href>" DEBUG_CR
                      "<D:propstat><D:prop>" DEBUG_CR
@@ -295,7 +298,7 @@ dav_error * dav_svn__merge_response(ap_filter_t *output,
   if (creationdate)
     {
       (void) ap_fputstrs(output, bb,
-                         "<D:creationdate>", 
+                         "<D:creationdate>",
                          apr_xml_quote_string(pool, creationdate->data, 1),
                          "</D:creationdate>" DEBUG_CR,
                          NULL);
@@ -303,8 +306,8 @@ dav_error * dav_svn__merge_response(ap_filter_t *output,
   if (creator_displayname)
     {
       (void) ap_fputstrs(output, bb,
-                         "<D:creator-displayname>", 
-                         apr_xml_quote_string(pool, 
+                         "<D:creator-displayname>",
+                         apr_xml_quote_string(pool,
                                               creator_displayname->data, 1),
                          "</D:creator-displayname>" DEBUG_CR,
                          NULL);
@@ -324,22 +327,22 @@ dav_error * dav_svn__merge_response(ap_filter_t *output,
     {
       /* Now we need to generate responses for all the resources which
          changed.  This is done through a delta of the two roots.
-         
+
          Note that a directory is not marked when open_dir is seen
          (since it typically is used just for changing members in that
          directory); instead, we want for a property change (the only
          reason the client would need to fetch a new directory).
-         
+
          ### we probably should say something about the dirs, so that
          ### we can pass back the new version URL */
-      
+
       /* and go make me proud, boy! */
       serr = do_resources(repos, root, new_rev, output, bb, pool);
       if (serr != NULL)
         {
-          return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                                     "Error constructing resource list.",
-                                     repos->pool);
+          return dav_svn__convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                      "Error constructing resource list.",
+                                      repos->pool);
         }
     }
 
