@@ -2,7 +2,7 @@
  * main.c :  Main control function for svnserve
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -25,6 +25,7 @@
 #include <apr_network_io.h>
 #include <apr_signal.h>
 #include <apr_thread_proc.h>
+#include <apr_portable.h>
 
 #include <locale.h>
 
@@ -39,8 +40,14 @@
 #include "svn_repos.h"
 #include "svn_fs.h"
 #include "svn_version.h"
+#include "svn_io.h"
 
 #include "svn_private_config.h"
+#include "winservice.h"
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>   /* For getpid() */
+#endif
 
 #include "server.h"
 
@@ -54,10 +61,12 @@ enum connection_handling_mode {
 
 /* The mode in which to run svnserve */
 enum run_mode {
+  run_mode_unspecified,
   run_mode_inetd,
   run_mode_daemon,
   run_mode_tunnel,
-  run_mode_listen_once
+  run_mode_listen_once,
+  run_mode_service
 };
 
 #if APR_HAS_FORK
@@ -81,10 +90,38 @@ enum run_mode {
 
 #endif
 
+
+#ifdef WIN32
+static apr_os_sock_t winservice_svnserve_accept_socket = INVALID_SOCKET;
+
+/* The SCM calls this function (on an arbitrary thread, not the main()
+   thread!) when it wants to stop the service.
+
+   For now, our strategy is to close the listener socket, in order to
+   unblock main() and cause it to exit its accept loop.  We cannot use
+   apr_socket_close, because that function deletes the apr_socket_t
+   structure, as well as closing the socket handle.  If we called
+   apr_socket_close here, then main() will also call apr_socket_close,
+   resulting in a double-free.  This way, we just close the kernel
+   socket handle, which causes the accept() function call to fail,
+   which causes main() to clean up the socket.  So, memory gets freed
+   only once.
+
+   This isn't pretty, but it's better than a lot of other options.
+   Currently, there is no "right" way to shut down svnserve.
+
+   We store the OS handle rather than a pointer to the apr_socket_t
+   structure in order to eliminate any possibility of illegal memory
+   access. */
+void winservice_notify_stop(void)
+{
+  if (winservice_svnserve_accept_socket != INVALID_SOCKET)
+    closesocket(winservice_svnserve_accept_socket);
+}
+#endif /* _WIN32 */
+
+
 /* Option codes and descriptions for svnserve.
- *
- * This must not have more than SVN_OPT_MAX_OPTIONS entries; if you
- * need more, increase that limit first.
  *
  * The entire list must be terminated with an entry of nulls.
  *
@@ -96,6 +133,9 @@ enum run_mode {
 #define SVNSERVE_OPT_FOREGROUND  258
 #define SVNSERVE_OPT_TUNNEL_USER 259
 #define SVNSERVE_OPT_VERSION     260
+#define SVNSERVE_OPT_PID_FILE    261
+#define SVNSERVE_OPT_SERVICE     262
+#define SVNSERVE_OPT_CONFIG_FILE 263
 
 static const apr_getopt_option_t svnserve__options[] =
   {
@@ -108,10 +148,11 @@ static const apr_getopt_option_t svnserve__options[] =
      N_("run in foreground (useful for debugging)")},
     {"help",             'h', 0, N_("display this help")},
     {"version",           SVNSERVE_OPT_VERSION, 0,
-     N_("show version information")},
+     N_("show program version information")},
     {"inetd",            'i', 0, N_("inetd mode")},
     {"root",             'r', 1, N_("root of directory to serve")},
-    {"read-only",        'R', 0, N_("deprecated; use repository config file")},
+    {"read-only",        'R', 0,
+     N_("force read only, overriding repository config file")},
     {"tunnel",           't', 0, N_("tunnel mode")},
     {"tunnel-user",      SVNSERVE_OPT_TUNNEL_USER, 1,
      N_("tunnel username (default is current uid's name)")},
@@ -119,6 +160,14 @@ static const apr_getopt_option_t svnserve__options[] =
     {"threads",          'T', 0, N_("use threads instead of fork")},
 #endif
     {"listen-once",      'X', 0, N_("listen once (useful for debugging)")},
+    {"config-file",      SVNSERVE_OPT_CONFIG_FILE, 1,
+     N_("read configuration from file ARG")},
+    {"pid-file",         SVNSERVE_OPT_PID_FILE, 1,
+     N_("write server process ID to file ARG")},
+#ifdef WIN32
+    {"service",          SVNSERVE_OPT_SERVICE, 0,
+     N_("run as a windows service (SCM only)")},
+#endif
     {0,                  0,   0, 0}
   };
 
@@ -138,7 +187,7 @@ static void help(apr_pool_t *pool)
 {
   apr_size_t i;
 
-  svn_error_clear(svn_cmdline_fputs(_("Usage: svnserve [options]\n"
+  svn_error_clear(svn_cmdline_fputs(_("usage: svnserve [options]\n"
                                       "\n"
                                       "Valid options:\n"),
                                     stdout, pool));
@@ -152,20 +201,25 @@ static void help(apr_pool_t *pool)
   exit(0);
 }
 
-static svn_error_t * version(apr_getopt_t *os, apr_pool_t *pool)
+static svn_error_t * version(apr_pool_t *pool)
 {
   const char *fs_desc_start
     = _("The following repository back-end (FS) modules are available:\n\n");
 
   svn_stringbuf_t *version_footer;
 
-  version_footer = svn_stringbuf_create (fs_desc_start, pool);
-  SVN_ERR (svn_fs_print_modules (version_footer, pool));
+  version_footer = svn_stringbuf_create(fs_desc_start, pool);
+  SVN_ERR(svn_fs_print_modules(version_footer, pool));
 
-  return svn_opt_print_help(os, "svnserve", TRUE, FALSE, version_footer->data,
+#ifdef SVN_HAVE_SASL
+  svn_stringbuf_appendcstr(version_footer,
+                           _("\nCyrus SASL authentication is available.\n"));
+#endif
+
+  return svn_opt_print_help(NULL, "svnserve", TRUE, FALSE, version_footer->data,
                             NULL, NULL, NULL, NULL, pool);
 }
-  
+
 
 #if APR_HAS_FORK
 static void sigchld_handler(int signo)
@@ -184,9 +238,12 @@ static apr_status_t redirect_stdout(void *arg)
 {
   apr_pool_t *pool = arg;
   apr_file_t *out_file, *err_file;
+  apr_status_t apr_err;
 
-  apr_file_open_stdout(&out_file, pool);
-  apr_file_open_stderr(&err_file, pool);
+  if ((apr_err = apr_file_open_stdout(&out_file, pool)))
+    return apr_err;
+  if ((apr_err = apr_file_open_stderr(&err_file, pool)))
+    return apr_err;
   return apr_file_dup2(out_file, err_file, pool);
 }
 
@@ -209,6 +266,25 @@ static void * APR_THREAD_FUNC serve_thread(apr_thread_t *tid, void *data)
 }
 #endif
 
+/* Write the PID of the current process as a decimal number, followed by a
+   newline to the file FILENAME, using POOL for temporary allocations. */
+static svn_error_t *write_pid_file(const char *filename, apr_pool_t *pool)
+{
+  apr_file_t *file;
+  const char *contents = apr_psprintf(pool, "%" APR_PID_T_FMT "\n",
+                                             getpid());
+
+  SVN_ERR(svn_io_file_open(&file, filename,
+                           APR_WRITE | APR_CREATE | APR_TRUNCATE,
+                           APR_OS_DEFAULT, pool));
+  SVN_ERR(svn_io_file_write_full(file, contents, strlen(contents), NULL,
+                                 pool));
+
+  SVN_ERR(svn_io_file_close(file, pool));
+
+  return SVN_NO_ERROR;
+}
+
 /* Version compatibility check */
 static svn_error_t *
 check_lib_versions(void)
@@ -228,9 +304,9 @@ check_lib_versions(void)
 }
 
 
-int main(int argc, const char *const *argv)
+int main(int argc, const char *argv[])
 {
-  enum run_mode run_mode;
+  enum run_mode run_mode = run_mode_unspecified;
   svn_boolean_t foreground = FALSE;
   apr_socket_t *sock, *usock;
   apr_file_t *in_file, *out_file;
@@ -239,7 +315,6 @@ int main(int argc, const char *const *argv)
   apr_pool_t *connection_pool;
   svn_error_t *err;
   apr_getopt_t *os;
-  char errbuf[256];
   int opt;
   serve_params_t params;
   const char *arg;
@@ -257,40 +332,43 @@ int main(int argc, const char *const *argv)
   const char *host = NULL;
   int family = APR_INET;
   int mode_opt_count = 0;
+  const char *config_filename = NULL;
+  const char *pid_filename = NULL;
+  svn_node_kind_t kind;
 
   /* Initialize the app. */
-  if (svn_cmdline_init("svn", stderr) != EXIT_SUCCESS)
+  if (svn_cmdline_init("svnserve", stderr) != EXIT_SUCCESS)
     return EXIT_FAILURE;
 
   /* Create our top-level pool. */
   pool = svn_pool_create(NULL);
 
+#ifdef SVN_HAVE_SASL
+  SVN_INT_ERR(cyrus_init(pool));
+#endif
+
   /* Check library versions */
   err = check_lib_versions();
   if (err)
-    {
-      svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-      svn_error_clear(err);
-      svn_pool_destroy(pool);
-      return EXIT_FAILURE;
-    }
+    return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
 
   /* Initialize the FS library. */
   err = svn_fs_initialize(pool);
   if (err)
-    {
-      svn_handle_error2(err, stderr, FALSE, "svnserve: ");
-      svn_error_clear(err);
-      svn_pool_destroy(pool);
-      return EXIT_FAILURE;
-    }
+    return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
 
-  apr_getopt_init(&os, pool, argc, argv);
+  err = svn_cmdline__getopt_init(&os, argc, argv, pool);
+  if (err)
+    return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
 
   params.root = "/";
   params.tunnel = FALSE;
   params.tunnel_user = NULL;
   params.read_only = FALSE;
+  params.cfg = NULL;
+  params.pwdb = NULL;
+  params.authzdb = NULL;
+
   while (1)
     {
       status = apr_getopt_long(os, svnserve__options, &opt, &arg);
@@ -305,13 +383,16 @@ int main(int argc, const char *const *argv)
           break;
 
         case SVNSERVE_OPT_VERSION:
-          SVN_INT_ERR(version(os, pool));
+          SVN_INT_ERR(version(pool));
           exit(0);
           break;
-          
+
         case 'd':
-          run_mode = run_mode_daemon;
-          mode_opt_count++;
+          if (run_mode != run_mode_daemon)
+            {
+              run_mode = run_mode_daemon;
+              mode_opt_count++;
+            }
           break;
 
         case SVNSERVE_OPT_FOREGROUND:
@@ -319,8 +400,11 @@ int main(int argc, const char *const *argv)
           break;
 
         case 'i':
-          run_mode = run_mode_inetd;
-          mode_opt_count++;
+          if (run_mode != run_mode_inetd)
+            {
+              run_mode = run_mode_inetd;
+              mode_opt_count++;
+            }
           break;
 
         case SVNSERVE_OPT_LISTEN_PORT:
@@ -332,8 +416,11 @@ int main(int argc, const char *const *argv)
           break;
 
         case 't':
-          run_mode = run_mode_tunnel;
-          mode_opt_count++;
+          if (run_mode != run_mode_tunnel)
+            {
+              run_mode = run_mode_tunnel;
+              mode_opt_count++;
+            }
           break;
 
         case SVNSERVE_OPT_TUNNEL_USER:
@@ -341,38 +428,86 @@ int main(int argc, const char *const *argv)
           break;
 
         case 'X':
-          run_mode = run_mode_listen_once;
-          mode_opt_count++;
+          if (run_mode != run_mode_listen_once)
+            {
+              run_mode = run_mode_listen_once;
+              mode_opt_count++;
+            }
           break;
 
         case 'r':
           SVN_INT_ERR(svn_utf_cstring_to_utf8(&params.root, arg, pool));
+
+          err = svn_io_check_resolved_path(params.root, &kind, pool);
+          if (err)
+            return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
+          if (kind != svn_node_dir)
+            {
+              svn_error_clear
+                (svn_cmdline_fprintf
+                   (stderr, pool,
+                    _("svnserve: Root path '%s' does not exist "
+                      "or is not a directory.\n"), params.root));
+              return EXIT_FAILURE;
+            }
+
           params.root = svn_path_internal_style(params.root, pool);
           SVN_INT_ERR(svn_path_get_absolute(&params.root, params.root, pool));
           break;
 
         case 'R':
           params.read_only = TRUE;
-          svn_error_clear
-            (svn_cmdline_fprintf
-               (stderr, pool,
-                _("Warning: -R is deprecated.\n"
-                  "Anonymous access is now read-only by default.\n"
-                  "To change, use conf/svnserve.conf in repos:\n"
-                  "  [general]\n"
-                  "  anon-access = read|write|none (default read)\n"
-                  "  auth-access = read|write|none (default write)\n"
-                  "Forcing all access to read-only for now\n")));
           break;
 
         case 'T':
           handling_mode = connection_mode_thread;
           break;
+
+#ifdef WIN32
+        case SVNSERVE_OPT_SERVICE:
+          if (run_mode != run_mode_service)
+            {
+              run_mode = run_mode_service;
+              mode_opt_count++;
+            }
+          break;
+#endif
+
+        case SVNSERVE_OPT_CONFIG_FILE:
+          SVN_INT_ERR(svn_utf_cstring_to_utf8(&config_filename, arg, pool));
+          config_filename = svn_path_internal_style(config_filename, pool);
+          SVN_INT_ERR(svn_path_get_absolute(&config_filename, config_filename,
+                                            pool));
+          break;
+
+        case SVNSERVE_OPT_PID_FILE:
+          SVN_INT_ERR(svn_utf_cstring_to_utf8(&pid_filename, arg, pool));
+          pid_filename = svn_path_internal_style(pid_filename, pool);
+          SVN_INT_ERR(svn_path_get_absolute(&pid_filename, pid_filename,
+                                            pool));
+          break;
+
         }
     }
   if (os->ind != argc)
     usage(argv[0], pool);
 
+  if (mode_opt_count != 1)
+    {
+      svn_error_clear(svn_cmdline_fputs
+                      (_("You must specify exactly one of -d, -i, -t or -X.\n"),
+                       stderr, pool));
+      usage(argv[0], pool);
+    }
+
+  /* If a configuration file is specified, load it and any referenced
+   * password and authorization files. */
+  if (config_filename)
+      SVN_INT_ERR(load_configs(&params.cfg, &params.pwdb, &params.authzdb,
+                               config_filename, TRUE,
+                               svn_path_dirname(config_filename, pool),
+                               pool));
+
   if (params.tunnel_user && run_mode != run_mode_tunnel)
     {
       svn_error_clear
@@ -382,29 +517,84 @@ int main(int argc, const char *const *argv)
       exit(1);
     }
 
-  if (mode_opt_count != 1)
-    {
-      svn_error_clear(svn_cmdline_fputs
-                      (_("You must specify exactly one of -d, -i, -t or -X.\n"),
-                       stderr, pool));
-      usage(argv[0], pool);
-    }
-
   if (run_mode == run_mode_inetd || run_mode == run_mode_tunnel)
     {
       params.tunnel = (run_mode == run_mode_tunnel);
       apr_pool_cleanup_register(pool, pool, apr_pool_cleanup_null,
                                 redirect_stdout);
-      apr_file_open_stdin(&in_file, pool);
-      apr_file_open_stdout(&out_file, pool);
+      status = apr_file_open_stdin(&in_file, pool);
+      if (status)
+        {
+          err = svn_error_wrap_apr(status, _("Can't open stdin"));
+          return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
+        }
+
+      status = apr_file_open_stdout(&out_file, pool);
+      if (status)
+        {
+          err = svn_error_wrap_apr(status, _("Can't open stdout"));
+          return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
+        }
+
       conn = svn_ra_svn_create_conn(NULL, in_file, out_file, pool);
       svn_error_clear(serve(conn, &params, pool));
       exit(0);
     }
- 
+
+#ifdef WIN32
+  /* If svnserve needs to run as a Win32 service, then we need to
+     coordinate with the Service Control Manager (SCM) before
+     continuing.  This function call registers the svnserve.exe
+     process with the SCM, waits for the "start" command from the SCM
+     (which will come very quickly), and confirms that those steps
+     succeeded.
+
+     After this call succeeds, the service is free to run.  At some
+     point in the future, the SCM will send a message to the service,
+     requesting that it stop.  This is translated into a call to
+     winservice_notify_stop().  The service is then responsible for
+     cleanly terminating.
+
+     We need to do this before actually starting the service logic
+     (opening files, sockets, etc.) because the SCM wants you to
+     connect *first*, then do your service-specific logic.  If the
+     service process takes too long to connect to the SCM, then the
+     SCM will decide that the service is busted, and will give up on
+     it.
+     */
+  if (run_mode == run_mode_service)
+    {
+      err = winservice_start();
+      if (err)
+        {
+          svn_handle_error2(err, stderr, FALSE, "svnserve: ");
+
+          /* This is the most common error.  It means the user started
+             svnserve from a shell, and specified the --service
+             argument.  svnserve cannot be started, as a service, in
+             this way.  The --service argument is valid only valid if
+             svnserve is started by the SCM. */
+          if (err->apr_err ==
+              APR_FROM_OS_ERROR(ERROR_FAILED_SERVICE_CONTROLLER_CONNECT))
+            {
+              svn_error_clear(svn_cmdline_fprintf(stderr, pool,
+                  _("svnserve: The --service flag is only valid if the"
+                    " process is started by the Service Control Manager.\n")));
+            }
+
+          svn_error_clear(err);
+          exit(1);
+        }
+
+      /* The service is now in the "starting" state.  Before the SCM will
+         consider the service "started", this thread must call the
+         winservice_running() function. */
+    }
+#endif /* WIN32 */
+
   /* Make sure we have IPV6 support first before giving apr_sockaddr_info_get
      APR_UNSPEC, because it may give us back an IPV6 address even if we can't
-     create IPV6 sockets. */  
+     create IPV6 sockets. */
 
 #if APR_HAVE_IPV6
 #ifdef MAX_SECS_TO_LINGER
@@ -414,21 +604,18 @@ int main(int argc, const char *const *argv)
   status = apr_socket_create(&sock, APR_INET6, SOCK_STREAM, APR_PROTO_TCP,
                              pool);
 #endif
-  if (status == 0)   
+  if (status == 0)
     {
       apr_socket_close(sock);
       family = APR_UNSPEC;
     }
 #endif
-  
+
   status = apr_sockaddr_info_get(&sa, host, family, port, 0, pool);
   if (status)
     {
-      svn_error_clear
-        (svn_cmdline_fprintf
-           (stderr, pool, _("Can't get address info: %s\n"),
-            apr_strerror(status, errbuf, sizeof(errbuf))));
-      exit(1);
+      err = svn_error_wrap_apr(status, _("Can't get address info"));
+      return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
     }
 
 
@@ -441,11 +628,8 @@ int main(int argc, const char *const *argv)
 #endif
   if (status)
     {
-      svn_error_clear
-        (svn_cmdline_fprintf
-           (stderr, pool, _("Can't create server socket: %s\n"),
-            apr_strerror(status, errbuf, sizeof(errbuf))));
-      exit(1);
+      err = svn_error_wrap_apr(status, _("Can't create server socket"));
+      return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
     }
 
   /* Prevents "socket in use" errors when server is killed and quickly
@@ -455,11 +639,8 @@ int main(int argc, const char *const *argv)
   status = apr_socket_bind(sock, sa);
   if (status)
     {
-      svn_error_clear
-        (svn_cmdline_fprintf
-           (stderr, pool, _("Can't bind server socket: %s\n"),
-            apr_strerror(status, errbuf, sizeof(errbuf))));
-      exit(1);
+      err = svn_error_wrap_apr(status, _("Can't bind server socket"));
+      return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
     }
 
   apr_socket_listen(sock, 7);
@@ -476,8 +657,33 @@ int main(int argc, const char *const *argv)
   apr_signal(SIGPIPE, SIG_IGN);
 #endif
 
+#ifdef SIGXFSZ
+  /* Disable SIGXFSZ generation for the platforms that have it, otherwise
+   * working with large files when compiled against an APR that doesn't have
+   * large file support will crash the program, which is uncool. */
+  apr_signal(SIGXFSZ, SIG_IGN);
+#endif
+
+  if (pid_filename)
+    SVN_INT_ERR(write_pid_file(pid_filename, pool));
+
+#ifdef WIN32
+  status = apr_os_sock_get(&winservice_svnserve_accept_socket, sock);
+  if (status)
+    winservice_svnserve_accept_socket = INVALID_SOCKET;
+
+  /* At this point, the service is "running".  Notify the SCM. */
+  if (run_mode == run_mode_service)
+    winservice_running();
+#endif
+
   while (1)
     {
+#ifdef WIN32
+      if (winservice_is_stopping())
+        return ERROR_SUCCESS;
+#endif
+
       /* Non-standard pool handling.  The main thread never blocks to join
          the connection threads so it cannot clean up after each one.  So
          separate pools, that can be cleared at thread exit, are used */
@@ -498,11 +704,9 @@ int main(int argc, const char *const *argv)
         }
       if (status)
         {
-          svn_error_clear
-            (svn_cmdline_fprintf
-             (stderr, pool, _("Can't accept client connection: %s\n"),
-              apr_strerror(status, errbuf, sizeof(errbuf))));
-          exit(1);
+          err = svn_error_wrap_apr
+            (status, _("Can't accept client connection"));
+          return svn_cmdline_handle_exit_error(err, pool, "svnserve: ");
         }
 
       conn = svn_ra_svn_create_conn(usock, NULL, NULL, connection_pool);
@@ -553,19 +757,17 @@ int main(int argc, const char *const *argv)
           status = apr_threadattr_create(&tattr, connection_pool);
           if (status)
             {
-              svn_error_clear
-                (svn_cmdline_fprintf
-                 (stderr, pool, _("Can't create threadattr: %s\n"),
-                  apr_strerror(status, errbuf, sizeof(errbuf))));
+              err = svn_error_wrap_apr(status, _("Can't create threadattr"));
+              svn_handle_error2(err, stderr, FALSE, "svnserve: ");
+              svn_error_clear(err);
               exit(1);
             }
           status = apr_threadattr_detach_set(tattr, 1);
           if (status)
             {
-              svn_error_clear
-                (svn_cmdline_fprintf
-                 (stderr, pool, _("Can't set detached state: %s\n"),
-                  apr_strerror(status, errbuf, sizeof(errbuf))));
+              err = svn_error_wrap_apr(status, _("Can't set detached state"));
+              svn_handle_error2(err, stderr, FALSE, "svnserve: ");
+              svn_error_clear(err);
               exit(1);
             }
           thread_data = apr_palloc(connection_pool, sizeof(*thread_data));
@@ -576,10 +778,9 @@ int main(int argc, const char *const *argv)
                                      connection_pool);
           if (status)
             {
-              svn_error_clear
-                (svn_cmdline_fprintf
-                 (stderr, pool, _("Can't create thread: %s\n"),
-                  apr_strerror(status, errbuf, sizeof(errbuf))));
+              err = svn_error_wrap_apr(status, _("Can't create thread"));
+              svn_handle_error2(err, stderr, FALSE, "svnserve: ");
+              svn_error_clear(err);
               exit(1);
             }
 #endif
@@ -592,5 +793,5 @@ int main(int argc, const char *const *argv)
         }
     }
 
-  return 0;
+  /* NOTREACHED */
 }
