@@ -2,11 +2,11 @@
 #
 #  trans_tests.py:  testing eol conversion and keyword substitution
 #
-#  Subversion is a tool for revision control. 
+#  Subversion is a tool for revision control.
 #  See http://subversion.tigris.org for more information.
-#    
+#
 # ====================================================================
-# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2006 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -17,7 +17,7 @@
 ######################################################################
 
 # General modules
-import string, sys, os.path, re
+import os, re
 
 # Our testing module
 import svntest
@@ -38,7 +38,7 @@ Item = svntest.wc.StateItem
 # status level 1:
 #    enable translation, status
 #    (now throw local text mods into the picture)
-#   
+#
 # commit level 1:
 #    enable translation, commit
 #    (now throw local text mods into the picture)
@@ -88,11 +88,11 @@ def check_keywords(actual_kw, expected_kw, name):
       print '%s item %s, Expected: %s' % (name, i, expected_kw[i][:-1])
       print '%s item %s, Got:      %s' % (name, i, actual_kw[i][:-1])
       raise svntest.Failure
- 
+
 def setup_working_copy(wc_dir, value_len):
   """Setup a standard test working copy, then create (but do not add)
   various files for testing translation."""
-  
+
   global author_rev_unexp_path
   global author_rev_exp_path
   global url_unexp_path
@@ -126,19 +126,19 @@ def setup_working_copy(wc_dir, value_len):
   id_with_space_path = os.path.join(wc_dir, 'id with space')
   id_exp_with_dollar_path = os.path.join(wc_dir, 'id_exp with_$_sign')
 
-  svntest.main.file_append (author_rev_unexp_path, "$Author$\n$Rev$")
-  svntest.main.file_append (author_rev_exp_path, "$Author: blah $\n$Rev: 0 $")
-  svntest.main.file_append (url_unexp_path, "$URL$")
-  svntest.main.file_append (url_exp_path, "$URL: blah $")
-  svntest.main.file_append (id_unexp_path, "$Id$")
-  svntest.main.file_append (id_exp_path, "$Id: blah $")
-  svntest.main.file_append (bogus_keywords_path, "$Arthur$\n$Rev0$")
-  svntest.main.file_append (embd_author_rev_unexp_path,
-                            "one\nfish\n$Author$ two fish\n red $Rev$\n fish")
-  svntest.main.file_append (embd_author_rev_exp_path,
-                            "blue $Author: blah $ fish$Rev: 0 $\nI fish")
-  svntest.main.file_append (embd_bogus_keywords_path,
-                            "you fish $Arthur$then\n we$Rev0$ \n\nchew fish")
+  svntest.main.file_append(author_rev_unexp_path, "$Author$\n$Rev$")
+  svntest.main.file_append(author_rev_exp_path, "$Author: blah $\n$Rev: 0 $")
+  svntest.main.file_append(url_unexp_path, "$URL$")
+  svntest.main.file_append(url_exp_path, "$URL: blah $")
+  svntest.main.file_append(id_unexp_path, "$Id$")
+  svntest.main.file_append(id_exp_path, "$Id: blah $")
+  svntest.main.file_append(bogus_keywords_path, "$Arthur$\n$Rev0$")
+  svntest.main.file_append(embd_author_rev_unexp_path,
+                           "one\nfish\n$Author$ two fish\n red $Rev$\n fish")
+  svntest.main.file_append(embd_author_rev_exp_path,
+                           "blue $Author: blah $ fish$Rev: 0 $\nI fish")
+  svntest.main.file_append(embd_bogus_keywords_path,
+                           "you fish $Arthur$then\n we$Rev0$ \n\nchew fish")
 
   keyword_test_targets = [
     # User tries to shoot him or herself on the foot
@@ -159,10 +159,10 @@ def setup_working_copy(wc_dir, value_len):
     ]
 
   for i in keyword_test_targets:
-    svntest.main.file_append (fixed_length_keywords_path, i)
+    svntest.main.file_append(fixed_length_keywords_path, i)
 
-  svntest.main.file_append (id_with_space_path, "$Id$")
-  svntest.main.file_append (id_exp_with_dollar_path, 
+  svntest.main.file_append(id_with_space_path, "$Id$")
+  svntest.main.file_append(id_exp_with_dollar_path,
                    "$Id: id_exp with_$_sign 1 2006-06-10 11:10:00Z jrandom $")
 
 
@@ -203,8 +203,8 @@ def keywords_from_birth(sbox):
     url_expand_test_data = canonical_repo_url + '/fixed_length_keywords'
   else:
     url_expand_test_data = canonical_repo_url + 'fixed_length_keywords'
-  
-  setup_working_copy (wc_dir, len(url_expand_test_data))
+
+  setup_working_copy(wc_dir, len(url_expand_test_data))
 
   # Add all the files
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
@@ -224,32 +224,32 @@ def keywords_from_birth(sbox):
     'id_exp with_$_sign' : Item(status='A ', wc_rev=0),
     })
 
-  svntest.main.run_svn (None, 'add', author_rev_unexp_path)
-  svntest.main.run_svn (None, 'add', author_rev_exp_path)
-  svntest.main.run_svn (None, 'add', url_unexp_path)
-  svntest.main.run_svn (None, 'add', url_exp_path)
-  svntest.main.run_svn (None, 'add', id_unexp_path)
-  svntest.main.run_svn (None, 'add', id_exp_path)
-  svntest.main.run_svn (None, 'add', bogus_keywords_path)
-  svntest.main.run_svn (None, 'add', embd_author_rev_unexp_path)
-  svntest.main.run_svn (None, 'add', embd_author_rev_exp_path)
-  svntest.main.run_svn (None, 'add', embd_bogus_keywords_path)
-  svntest.main.run_svn (None, 'add', fixed_length_keywords_path)
-  svntest.main.run_svn (None, 'add', id_with_space_path)
-  svntest.main.run_svn (None, 'add', id_exp_with_dollar_path)
+  svntest.main.run_svn(None, 'add', author_rev_unexp_path)
+  svntest.main.run_svn(None, 'add', author_rev_exp_path)
+  svntest.main.run_svn(None, 'add', url_unexp_path)
+  svntest.main.run_svn(None, 'add', url_exp_path)
+  svntest.main.run_svn(None, 'add', id_unexp_path)
+  svntest.main.run_svn(None, 'add', id_exp_path)
+  svntest.main.run_svn(None, 'add', bogus_keywords_path)
+  svntest.main.run_svn(None, 'add', embd_author_rev_unexp_path)
+  svntest.main.run_svn(None, 'add', embd_author_rev_exp_path)
+  svntest.main.run_svn(None, 'add', embd_bogus_keywords_path)
+  svntest.main.run_svn(None, 'add', fixed_length_keywords_path)
+  svntest.main.run_svn(None, 'add', id_with_space_path)
+  svntest.main.run_svn(None, 'add', id_exp_with_dollar_path)
 
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
   # Add the keyword properties.
-  keywords_on (author_rev_unexp_path)
-  keywords_on (url_unexp_path)
-  keywords_on (url_exp_path)
-  keywords_on (id_unexp_path)
-  keywords_on (id_exp_path)
-  keywords_on (embd_author_rev_exp_path)
-  keywords_on (fixed_length_keywords_path)
-  keywords_on (id_with_space_path)
-  keywords_on (id_exp_with_dollar_path)
+  keywords_on(author_rev_unexp_path)
+  keywords_on(url_unexp_path)
+  keywords_on(url_exp_path)
+  keywords_on(id_unexp_path)
+  keywords_on(id_exp_path)
+  keywords_on(embd_author_rev_exp_path)
+  keywords_on(fixed_length_keywords_path)
+  keywords_on(id_with_space_path)
+  keywords_on(id_exp_with_dollar_path)
 
   # Commit.
   expected_output = svntest.wc.State(wc_dir, {
@@ -268,9 +268,8 @@ def keywords_from_birth(sbox):
     'id_exp with_$_sign' : Item(verb='Adding'),
     })
 
-  svntest.actions.run_and_verify_commit (wc_dir, expected_output,
-                                         None, None,
-                                         None, None, None, None, wc_dir)
+  svntest.actions.run_and_verify_commit(wc_dir, expected_output,
+                                        None, None, wc_dir)
 
   # Make sure the unexpanded URL keyword got expanded correctly.
   fp = open(url_unexp_path, 'r')
@@ -307,7 +306,7 @@ def keywords_from_birth(sbox):
     print "Id expansion failed for", id_exp_path
     raise svntest.Failure
   fp.close()
-  
+
   # Check fixed length keywords.
   kw_workingcopy = [
     '$URL::$\n',
@@ -342,13 +341,13 @@ def keywords_from_birth(sbox):
     '$URL:: %sx$\n' % (' ' * len(url_expand_test_data)),
     '$URL::x%sx$\n' % (' ' * len(url_expand_test_data))
     ]
-  
+
   fp = open(os.path.join(wc_dir, svntest.main.get_admin_name(),
                          'text-base', 'fixed_length_keywords.svn-base'), 'r')
   actual_textbase_kw = fp.readlines()
   fp.close()
   check_keywords(actual_textbase_kw, kw_textbase, "text base")
-  
+
   # Check the Id keyword for filename with spaces.
   fp = open(id_with_space_path, 'r')
   lines = fp.readlines()
@@ -362,10 +361,10 @@ def keywords_from_birth(sbox):
   fp = open(id_exp_with_dollar_path, 'r')
   lines = fp.readlines()
   if not ((len(lines) == 1)
-          and (re.match("\$Id: .*id_exp with_\$_sign [^$]* jrandom \$", 
+          and (re.match("\$Id: .*id_exp with_\$_sign [^$]* jrandom \$",
                         lines[0]))):
     print "Id expansion failed for", id_exp_with_dollar_path
-    
+
     raise svntest.Failure
   fp.close()
 
@@ -411,9 +410,7 @@ def update_modified_with_translation(sbox):
 
   # Replace contents of rho and set eol translation to 'native'
   rho_path = os.path.join(wc_dir, 'A', 'D', 'G', 'rho')
-  f = open(rho_path, "w")
-  f.write("1\n2\n3\n4\n5\n6\n7\n8\n9\n")
-  f.close()
+  svntest.main.file_write(rho_path, "1\n2\n3\n4\n5\n6\n7\n8\n9\n")
   svntest.actions.run_and_verify_svn(None, None, [],
                                      'propset', 'svn:eol-style', 'native',
                                      rho_path)
@@ -423,38 +420,30 @@ def update_modified_with_translation(sbox):
     'A/D/G/rho' : Item(verb='Sending'),
     })
 
-  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
-  expected_status.tweak(wc_rev=1)
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   # rho has props
   expected_status.tweak('A/D/G/rho', wc_rev=2, status='  ')
 
   # Commit revision 2:  it has the new rho.
-  svntest.actions.run_and_verify_commit (wc_dir,
-                                         expected_output,
-                                         expected_status,
-                                         None, None, None, None, None,
-                                         rho_path)
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        expected_output,
+                                        expected_status,
+                                        None, rho_path)
 
   # Change rho again
-  f = open(rho_path, "w")
-  f.write("1\n2\n3\n4\n4.5\n5\n6\n7\n8\n9\n")
-  f.close()
+  svntest.main.file_write(rho_path, "1\n2\n3\n4\n4.5\n5\n6\n7\n8\n9\n")
 
-  # Commit revision 3 
-  expected_status = svntest.actions.get_virginal_state(wc_dir, 3)
-  expected_status.tweak(wc_rev=1)
+  # Commit revision 3
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.tweak('A/D/G/rho', wc_rev=3, status='  ')
 
-  svntest.actions.run_and_verify_commit (wc_dir,
-                                         expected_output,
-                                         expected_status,
-                                         None, None, None, None, None,
-                                         rho_path)
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        expected_output,
+                                        expected_status,
+                                        None, rho_path)
 
   # Locally modify rho again.
-  f = open(rho_path, "w")
-  f.write("1\n2\n3\n4\n4.5\n5\n6\n7\n8\n9\n10\n")
-  f.close()
+  svntest.main.file_write(rho_path, "1\n2\n3\n4\n4.5\n5\n6\n7\n8\n9\n10\n")
 
   # Prepare trees for an update to rev 1.
   expected_output = svntest.wc.State(wc_dir, {
@@ -462,22 +451,23 @@ def update_modified_with_translation(sbox):
     })
 
   expected_disk = svntest.main.greek_state.copy()
-  expected_disk.tweak('A/D/G/rho', contents="""<<<<<<< .mine
-1
-2
-3
-4
-4.5
-5
-6
-7
-8
-9
-10
-=======
-This is the file 'rho'.
->>>>>>> .r1
-""")
+  expected_disk.tweak('A/D/G/rho',
+                      contents="\n".join(["<<<<<<< .mine",
+                                          "1",
+                                          "2",
+                                          "3",
+                                          "4",
+                                          "4.5",
+                                          "5",
+                                          "6",
+                                          "7",
+                                          "8",
+                                          "9",
+                                          "10",
+                                          "=======",
+                                          "This is the file 'rho'.",
+                                          ">>>>>>> .r1",
+                                          ""]))
 
   # Updating back to revision 1 should not error; the merge should
   # work, with eol-translation turned on.
@@ -503,7 +493,7 @@ This is the file 'rho'.
 
 def eol_change_is_text_mod(sbox):
   "committing eol-style change forces text send"
-  
+
   sbox.build()
 
   wc_dir = sbox.wc_dir
@@ -519,9 +509,10 @@ def eol_change_is_text_mod(sbox):
 
   # commit the file
   svntest.actions.run_and_verify_svn(None, None, [], 'add', foo_path)
-  svntest.actions.run_and_verify_svn(None, None, [], 'ci', '-m', 'log msg',
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg',
                                      foo_path)
-  
+
   if svntest.main.windows:
     svntest.actions.run_and_verify_svn(None, None, [], 'propset',
                                        'svn:eol-style', 'LF', foo_path)
@@ -537,22 +528,20 @@ def eol_change_is_text_mod(sbox):
                                      'ci', '-m', 'log msg', foo_path)
 
   # check 2: do the files have the right contents now?
-  f = open(foo_path, 'rb')
-  contents = f.read()
-  f.close()
+  contents = svntest.main.file_read(foo_path, 'rb')
   if svntest.main.windows:
     if contents != "1\n2\n3\n4\n5\n6\n7\n8\n9\n":
       raise svntest.Failure
   else:
     if contents != "1\r\n2\r\n3\r\n4\r\n5\r\n6\r\n7\r\n8\r\n9\r\n":
       raise svntest.Failure
-  f = open(os.path.join(wc_dir, svntest.main.get_admin_name(),
-                        'text-base', 'foo.svn-base'), 'rb')
-  base_contents = f.read()
-  f.close()
+
+  foo_base_path = os.path.join(wc_dir, svntest.main.get_admin_name(),
+                               'text-base', 'foo.svn-base')
+  base_contents = svntest.main.file_read(foo_base_path, 'rb')
   if contents != base_contents:
     raise svntest.Failure
-  
+
 #----------------------------------------------------------------------
 # Regression test for issue #1151.  A single file in a directory
 # didn't get keywords expanded on checkout.
@@ -566,12 +555,12 @@ def keyword_expanded_on_checkout(sbox):
   # The bug didn't occur if there were multiple files in the
   # directory, so setup an empty directory.
   Z_path = os.path.join(wc_dir, 'Z')
-  svntest.actions.run_and_verify_svn (None, None, [], 'mkdir', Z_path)
-  
+  svntest.actions.run_and_verify_svn(None, None, [], 'mkdir', Z_path)
+
   # Add the file that has the keyword to be expanded
   url_path = os.path.join(Z_path, 'url')
-  svntest.main.file_append (url_path, "$URL$")
-  svntest.actions.run_and_verify_svn (None, None, [], 'add', url_path)
+  svntest.main.file_append(url_path, "$URL$")
+  svntest.actions.run_and_verify_svn(None, None, [], 'add', url_path)
   keywords_on(url_path)
 
   svntest.actions.run_and_verify_svn(None, None, [],
@@ -579,11 +568,9 @@ def keyword_expanded_on_checkout(sbox):
 
   other_wc_dir = sbox.add_wc_path('other')
   other_url_path = os.path.join(other_wc_dir, 'Z', 'url')
-  svntest.actions.run_and_verify_svn (None, None, [], 'checkout',
-                                      '--username', svntest.main.wc_author,
-                                      '--password', svntest.main.wc_passwd,
-                                      svntest.main.current_repo_url,
-                                      other_wc_dir)
+  svntest.actions.run_and_verify_svn(None, None, [], 'checkout',
+                                     sbox.repo_url,
+                                     other_wc_dir)
 
   # Check keyword got expanded (and thus the mkdir, add, ps, commit
   # etc. worked)
@@ -606,7 +593,7 @@ def cat_keyword_expansion(sbox):
   lambda_path = os.path.join(wc_dir, 'A', 'B', 'lambda')
 
   # Set up A/mu to do $Rev$ keyword expansion
-  svntest.main.file_append (mu_path , "$Rev$\n$Author$")
+  svntest.main.file_append(mu_path , "$Rev$\n$Author$")
   svntest.actions.run_and_verify_svn(None, None, [],
                                      'propset', 'svn:keywords', 'Rev Author',
                                      mu_path)
@@ -614,23 +601,21 @@ def cat_keyword_expansion(sbox):
   expected_output = wc.State(wc_dir, {
     'A/mu' : Item(verb='Sending'),
     })
-  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
-  expected_status.tweak(wc_rev=1)
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.tweak('A/mu', wc_rev=2)
-  svntest.actions.run_and_verify_commit (wc_dir,
-                                         expected_output, expected_status,
-                                         None, None, None, None, None,
-                                         wc_dir)
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        expected_output, expected_status,
+                                        None, wc_dir)
 
   # Change the author to value which will get truncated on expansion
-  full_author = "x" * 400 
+  full_author = "x" * 400
   key_author = "x" * 244
-  svntest.actions.enable_revprop_changes(svntest.main.current_repo_dir)
-  svntest.actions.run_and_verify_svn("", None, [],
+  svntest.actions.enable_revprop_changes(sbox.repo_dir)
+  svntest.actions.run_and_verify_svn(None, None, [],
                                      'propset', '--revprop', '-r2',
                                      'svn:author', full_author,
                                      sbox.wc_dir)
-  svntest.actions.run_and_verify_svn("", [ full_author ], [],
+  svntest.actions.run_and_verify_svn(None, [ full_author ], [],
                                      'propget', '--revprop', '-r2',
                                      'svn:author', '--strict',
                                      sbox.wc_dir)
@@ -643,19 +628,18 @@ def cat_keyword_expansion(sbox):
     'A/B/lambda' : Item(verb='Sending'),
     })
   expected_status.tweak('A/B/lambda', wc_rev=3)
-  svntest.actions.run_and_verify_commit (wc_dir,
-                                         expected_output, expected_status,
-                                         None, None, None, None, None,
-                                         wc_dir)
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        expected_output, expected_status,
+                                        None, wc_dir)
 
   # At one stage the keywords were expanded to values for the requested
   # revision, not to those committed revision
-  svntest.actions.run_and_verify_svn (None,
-                                      [ "This is the file 'mu'.\n",
-                                        "$Rev: 2 $\n",
-                                        "$Author: " + key_author + " $"], [],
-                                      'cat', '-r', 'HEAD', mu_path)
-  
+  svntest.actions.run_and_verify_svn(None,
+                                     [ "This is the file 'mu'.\n",
+                                       "$Rev: 2 $\n",
+                                       "$Author: " + key_author + " $"], [],
+                                     'cat', '-r', 'HEAD', mu_path)
+
 
 #----------------------------------------------------------------------
 def copy_propset_commit(sbox):
@@ -682,10 +666,9 @@ def copy_propset_commit(sbox):
     'A/mu2' : Item(verb='Adding'),
     })
   expected_status.tweak('A/mu2', status='  ', wc_rev=2, copied=None)
-  svntest.actions.run_and_verify_commit (wc_dir,
-                                         expected_output, expected_status,
-                                         None, None, None, None, None,
-                                         wc_dir)
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        expected_output, expected_status,
+                                        None, wc_dir)
 
 #----------------------------------------------------------------------
 #      Create a greek tree, commit a keyword into one file,
@@ -700,17 +683,15 @@ def propset_commit_checkout_nocrash(sbox):
   mu_path = os.path.join(wc_dir, 'A', 'mu')
 
   # Put a keyword in A/mu, commit
-  svntest.main.file_append (mu_path, "$Rev$")
+  svntest.main.file_append(mu_path, "$Rev$")
   expected_output = wc.State(wc_dir, {
     'A/mu' : Item(verb='Sending'),
     })
-  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
-  expected_status.tweak(wc_rev=1)
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.tweak('A/mu', wc_rev=2)
   svntest.actions.run_and_verify_commit(wc_dir,
                                         expected_output, expected_status,
-                                        None, None, None, None, None,
-                                        wc_dir)
+                                        None, wc_dir)
 
   # Set property to do keyword expansion on A/mu, commit.
   svntest.actions.run_and_verify_svn(None, None, [],
@@ -718,30 +699,26 @@ def propset_commit_checkout_nocrash(sbox):
   expected_output = wc.State(wc_dir, {
     'A/mu' : Item(verb='Sending'),
     })
-  expected_status = svntest.actions.get_virginal_state(wc_dir, 3)
-  expected_status.tweak(wc_rev=1)
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_status.tweak('A/mu', wc_rev=3)
   svntest.actions.run_and_verify_commit(wc_dir,
                                         expected_output, expected_status,
-                                        None, None, None, None, None,
-                                        wc_dir)
+                                        None, wc_dir)
 
   # Check out into another wc dir
   other_wc_dir = sbox.add_wc_path('other')
   mu_other_path = os.path.join(other_wc_dir, 'A', 'mu')
-  
-  svntest.actions.run_and_verify_svn (None, None, [], 'checkout',
-                                      '--username', svntest.main.wc_author,
-                                      '--password', svntest.main.wc_passwd,
-                                      svntest.main.current_repo_url,
-                                      other_wc_dir)
 
-  mu_other_contents = open(mu_other_path).read()
+  svntest.actions.run_and_verify_svn(None, None, [], 'checkout',
+                                     sbox.repo_url,
+                                     other_wc_dir)
+
+  mu_other_contents = svntest.main.file_read(mu_other_path)
   if mu_other_contents != "This is the file 'mu'.\n$Rev: 3 $":
     print "'%s' does not have the expected contents" % mu_other_path
     raise svntest.Failure
 
-  
+
 #----------------------------------------------------------------------
 #      Add the keyword property to a file, svn revert the file
 #      This should not display any error message
@@ -766,7 +743,7 @@ def propset_revert_noerror(sbox):
   expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
   svntest.actions.run_and_verify_status(wc_dir, expected_status)
 
-  
+
 ########################################################################
 # Run the tests
 
@@ -783,7 +760,7 @@ test_list = [ None,
               cat_keyword_expansion,
               copy_propset_commit,
               propset_commit_checkout_nocrash,
-              propset_revert_noerror, 
+              propset_revert_noerror,
              ]
 
 if __name__ == '__main__':
