@@ -2,10 +2,10 @@
 #
 # mailer.py: send email describing a commit
 #
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.4.x/tools/hook-scripts/mailer/mailer.py $
-# $LastChangedDate: 2006-09-11 18:00:53 +0000 (Mon, 11 Sep 2006) $
-# $LastChangedBy: dlr $
-# $LastChangedRevision: 21431 $
+# $HeadURL: http://svn.collab.net/repos/svn/branches/1.5.x/tools/hook-scripts/mailer/mailer.py $
+# $LastChangedDate: 2007-09-25 15:15:57 +0000 (Tue, 25 Sep 2007) $
+# $LastChangedBy: cmpilato $
+# $LastChangedRevision: 26780 $
 #
 # USAGE: mailer.py commit      REPOS REVISION [CONFIG-FILE]
 #        mailer.py propchange  REPOS REVISION AUTHOR REVPROPNAME [CONFIG-FILE]
@@ -21,9 +21,8 @@
 #   in Subversion 1.2.0.  Its value is one of 'A', 'M' or 'D' to indicate
 #   if the property was added, modified or deleted, respectively.
 #
-#   This version of mailer.py requires the python bindings from
-#   subversion 1.2.0 or later.
-#
+#   See _MIN_SVN_VERSION below for which version of Subversion's Python
+#   bindings are required by this version of mailer.py.
 
 import os
 import sys
@@ -38,14 +37,32 @@ import tempfile
 import types
 import urllib
 
-import svn.fs
-import svn.delta
-import svn.repos
-import svn.core
+# Minimal version of Subversion's bindings required
+_MIN_SVN_VERSION = [1, 5, 0]
+
+# Import the Subversion Python bindings, making sure they meet our
+# minimum version requirements.
+try:
+  import svn.fs
+  import svn.delta
+  import svn.repos
+  import svn.core
+except ImportError:
+  sys.stderr.write(
+    "You need version %s or better of the Subversion Python bindings.\n" \
+    % string.join(map(lambda x: str(x), _MIN_SVN_VERSION), '.'))
+  sys.exit(1)
+if _MIN_SVN_VERSION > [svn.core.SVN_VER_MAJOR,
+                       svn.core.SVN_VER_MINOR,
+                       svn.core.SVN_VER_PATCH]:
+  sys.stderr.write(
+    "You need version %s or better of the Subversion Python bindings.\n" \
+    % string.join(map(lambda x: str(x), _MIN_SVN_VERSION), '.'))
+  sys.exit(1)
+
 
 SEPARATOR = '=' * 78
 
-
 def main(pool, cmd, config_fname, repos_dir, cmd_args):
   ### TODO:  Sanity check the incoming args
 
@@ -112,6 +129,11 @@ except AttributeError:
       rv = self.tochild.close() or rv
       return rv
 
+def remove_leading_slashes(path):
+  while path and path[0] == '/':
+    path = path[1:]
+  return path
+
 
 class OutputBase:
   "Abstract base class to formalize the inteface of output methods"
@@ -183,12 +205,31 @@ class MailedOutput(OutputBase):
     OutputBase.__init__(self, cfg, repos, prefix_param)
 
   def start(self, group, params):
-    # whitespace-separated list of addresses; split into a clean list:
-    self.to_addrs = \
-        filter(None, string.split(self.cfg.get('to_addr', group, params)))
+    # whitespace (or another character) separated list of addresses
+    # which must be split into a clean list
+    to_addr_in = self.cfg.get('to_addr', group, params)
+    # if list of addresses starts with '[.]'
+    # use the character between the square brackets as split char
+    # else use whitespaces
+    if len(to_addr_in) >= 3 and to_addr_in[0] == '[' \
+                            and to_addr_in[2] == ']':
+      self.to_addrs = \
+        filter(None, string.split(to_addr_in[3:], to_addr_in[1]))
+    else:
+      self.to_addrs = filter(None, string.split(to_addr_in))
     self.from_addr = self.cfg.get('from_addr', group, params) \
                      or self.repos.author or 'no_author'
+    # if the from_addr (also) starts with '[.]' (may happen if one
+    # map is used for both to_addr and from_addr) remove '[.]'
+    if len(self.from_addr) >= 3 and self.from_addr[0] == '[' \
+                                and self.from_addr[2] == ']':
+      self.from_addr = self.from_addr[3:]
     self.reply_to = self.cfg.get('reply_to', group, params)
+    # if the reply_to (also) starts with '[.]' (may happen if one
+    # map is used for both to_addr and reply_to) remove '[.]'
+    if len(self.reply_to) >= 3 and self.reply_to[0] == '[' \
+                               and self.reply_to[2] == ']':
+      self.reply_to = self.reply_to[3:]
 
   def mail_headers(self, group, params):
     subject = self.make_subject(group, params)
@@ -299,7 +340,8 @@ class Commit(Messenger):
     Messenger.__init__(self, pool, cfg, repos, 'commit_subject_prefix')
 
     # get all the changes and sort by path
-    editor = svn.repos.ChangeCollector(repos.fs_ptr, repos.root_this, self.pool)
+    editor = svn.repos.ChangeCollector(repos.fs_ptr, repos.root_this, \
+                                       self.pool)
     e_ptr, e_baton = svn.delta.make_editor(editor, self.pool)
     svn.repos.replay(repos.root_this, e_ptr, e_baton, self.pool)
 
@@ -445,7 +487,7 @@ def get_commondir(dirlist):
   a commondir is found, the dirlist returned is rooted in that
   commondir.  If no commondir is found, dirlist is returned unchanged,
   and commondir is the empty string."""
-  if len(dirlist) == 1 or '/' in dirlist:
+  if len(dirlist) < 2 or '/' in dirlist:
     commondir = ''
     newdirs = dirlist
   else:
@@ -583,7 +625,8 @@ class DiffURLSelections:
     # KeyError exceptions.
     params = self.params.copy()
     params['path'] = change.path and urllib.quote(change.path) or None
-    params['base_path'] = change.base_path and urllib.quote(change.base_path) or None
+    params['base_path'] = change.base_path and urllib.quote(change.base_path) \
+                          or None
     params['rev'] = repos_rev
     params['base_rev'] = change.base_rev
 
@@ -619,10 +662,12 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
   commit_url = cfg.get('commit_url', group, params_with_rev)
 
   # figure out the lists of changes outside the selected path-space
-  other_added_data = other_removed_data = other_modified_data = [ ]
+  other_added_data = other_replaced_data = other_deleted_data = \
+      other_modified_data = [ ]
   if len(paths) != len(changelist) and show_nonmatching_paths != 'no':
     other_added_data = generate_list('A', changelist, paths, False)
-    other_removed_data = generate_list('R', changelist, paths, False)
+    other_replaced_data = generate_list('R', changelist, paths, False)
+    other_deleted_data = generate_list('D', changelist, paths, False)
     other_modified_data = generate_list('M', changelist, paths, False)
 
   if len(paths) != len(changelist) and show_nonmatching_paths == 'yes':
@@ -638,11 +683,13 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
     log=repos.get_rev_prop(svn.core.SVN_PROP_REVISION_LOG) or '',
     commit_url=commit_url,
     added_data=generate_list('A', changelist, paths, True),
-    removed_data=generate_list('R', changelist, paths, True),
+    replaced_data=generate_list('R', changelist, paths, True),
+    deleted_data=generate_list('D', changelist, paths, True),
     modified_data=generate_list('M', changelist, paths, True),
     show_nonmatching_paths=show_nonmatching_paths,
     other_added_data=other_added_data,
-    other_removed_data=other_removed_data,
+    other_replaced_data=other_replaced_data,
+    other_deleted_data=other_deleted_data,
     other_modified_data=other_modified_data,
     diffs=DiffGenerator(changelist, paths, True, cfg, repos, date, group,
                         params, diffsels, diffurls, pool),
@@ -653,11 +700,13 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
 
 def generate_list(changekind, changelist, paths, in_paths):
   if changekind == 'A':
-    selection = lambda change: change.added
+    selection = lambda change: change.action == svn.repos.CHANGE_ACTION_ADD
   elif changekind == 'R':
-    selection = lambda change: change.path is None
+    selection = lambda change: change.action == svn.repos.CHANGE_ACTION_REPLACE
+  elif changekind == 'D':
+    selection = lambda change: change.action == svn.repos.CHANGE_ACTION_DELETE
   elif changekind == 'M':
-    selection = lambda change: not change.added and change.path is not None
+    selection = lambda change: change.action == svn.repos.CHANGE_ACTION_MODIFY
 
   items = [ ]
   for path, change in changelist:
@@ -667,8 +716,10 @@ def generate_list(changekind, changelist, paths, in_paths):
         is_dir=change.item_kind == svn.core.svn_node_dir,
         props_changed=change.prop_changes,
         text_changed=change.text_changed,
-        copied=change.added and change.base_path,
-        base_path=change.base_path,
+        copied=(change.action == svn.repos.CHANGE_ACTION_ADD \
+                or change.action == svn.repos.CHANGE_ACTION_REPLACE) \
+               and change.base_path,
+        base_path=remove_leading_slashes(change.base_path),
         base_rev=change.base_rev,
         )
       items.append(item)
@@ -727,9 +778,18 @@ class DiffGenerator:
       if self.paths.has_key(path) != self.in_paths:
         continue
 
+      if change.base_rev != -1:
+        svndate = self.repos.get_rev_prop(svn.core.SVN_PROP_REVISION_DATE,
+                                          change.base_rev)
+        ### pick a different date format?
+        base_date = time.ctime(svn.core.secs_from_timestr(svndate, self.pool))
+      else:
+        base_date = ''
+
       # figure out if/how to generate a diff
 
-      if not change.path:
+      base_path = remove_leading_slashes(change.base_path)
+      if change.action == svn.repos.CHANGE_ACTION_DELETE:
         # it was delete.
         kind = 'D'
 
@@ -739,31 +799,46 @@ class DiffGenerator:
         # show the diff?
         if self.diffsels.delete:
           diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
-                                 change.base_path, None, None, self.pool)
+                                 base_path, None, None, self.pool)
 
-          label1 = '%s\t%s' % (change.base_path, self.date)
-          label2 = '(empty file)'
+          label1 = '%s\t%s\t(r%s)' % (base_path, self.date, change.base_rev)
+          label2 = '/dev/null\t00:00:00 1970\t(deleted)'
           singular = True
 
-      elif change.added:
-        if change.base_path and (change.base_rev != -1):
-          # this file was copied.
-          kind = 'C'
+      elif change.action == svn.repos.CHANGE_ACTION_ADD \
+           or change.action == svn.repos.CHANGE_ACTION_REPLACE:
+        if base_path and (change.base_rev != -1):
 
           # any diff of interest?
           if change.text_changed:
+            # this file was copied and modified.
+            kind = 'W'
 
             # get the diff url, if any is specified
             diff_url = self.diffurls.get_copy_url(self.repos.rev, change)
 
             # show the diff?
-            if self.diffsels.copy:
+            if self.diffsels.modify:
               diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
-                                     change.base_path,
+                                     base_path,
                                      self.repos.root_this, change.path,
                                      self.pool)
-              label1 = change.base_path + '\t(original)'
-              label2 = '%s\t%s' % (change.path, self.date)
+              label1 = '%s\t%s\t(r%s, copy source)' \
+                       % (base_path, base_date, change.base_rev)
+              label2 = '%s\t%s\t(r%s)' \
+                       % (change.path, self.date, self.repos.rev)
+              singular = False
+          else:
+            # this file was copied.
+            kind = 'C'
+            if self.diffsels.copy:
+              diff = svn.fs.FileDiff(None, None, self.repos.root_this,
+                                     change.path, self.pool)
+              label1 = '/dev/null\t00:00:00 1970\t' \
+                       '(empty, because file is newly added)'
+              label2 = '%s\t%s\t(r%s, copy of r%s, %s)' \
+                       % (change.path, self.date, self.repos.rev, \
+                          change.base_rev, base_path)
               singular = False
         else:
           # the file was added.
@@ -776,8 +851,10 @@ class DiffGenerator:
           if self.diffsels.add:
             diff = svn.fs.FileDiff(None, None, self.repos.root_this,
                                    change.path, self.pool)
-            label1 = '(empty file)'
-            label2 = '%s\t%s' % (change.path, self.date)
+            label1 = '/dev/null\t00:00:00 1970\t' \
+                     '(empty, because file is newly added)'
+            label2 = '%s\t%s\t(r%s)' \
+                     % (change.path, self.date, self.repos.rev)
             singular = True
 
       elif not change.text_changed:
@@ -793,11 +870,13 @@ class DiffGenerator:
         # show the diff?
         if self.diffsels.modify:
           diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
-                                 change.base_path,
+                                 base_path,
                                  self.repos.root_this, change.path,
                                  self.pool)
-          label1 = change.base_path + '\t(original)'
-          label2 = '%s\t%s' % (change.path, self.date)
+          label1 = '%s\t%s\t(r%s)' \
+                   % (base_path, base_date, change.base_rev)
+          label2 = '%s\t%s\t(r%s)' \
+                   % (change.path, self.date, self.repos.rev)
           singular = False
 
       if diff:
@@ -816,7 +895,7 @@ class DiffGenerator:
       # return a data item for this diff
       return _data(
         path=change.path,
-        base_path=change.base_path,
+        base_path=base_path,
         base_rev=change.base_rev,
         diff=diff,
         diff_url=diff_url,
@@ -875,6 +954,9 @@ class DiffContent:
     else:
       ltype = 'U'
 
+    if line[-2] == '\r':
+      line=line[0:-2] + '\n' # remove carriage return
+
     return _data(
       raw=line,
       text=line[1:-1],  # remove indicator and newline
@@ -902,19 +984,21 @@ class TextCommitRenderer:
     else:
       w('\n')
 
-    w('Log:\n%s\n\n' % data.log)
+    w('Log:\n%s\n\n' % data.log.strip())
 
     # print summary sections
     self._render_list('Added', data.added_data)
-    self._render_list('Removed', data.removed_data)
+    self._render_list('Replaced', data.replaced_data)
+    self._render_list('Deleted', data.deleted_data)
     self._render_list('Modified', data.modified_data)
 
-    if data.other_added_data or data.other_removed_data \
-           or data.other_modified_data:
+    if data.other_added_data or data.other_replaced_data \
+           or data.other_deleted_data or data.other_modified_data:
       if data.show_nonmatching_paths:
         w('\nChanges in other areas also in this revision:\n')
         self._render_list('Added', data.other_added_data)
-        self._render_list('Removed', data.other_removed_data)
+        self._render_list('Replaced', data.other_replaced_data)
+        self._render_list('Deleted', data.other_deleted_data)
         self._render_list('Modified', data.other_modified_data)
       else:
         w('and changes in other areas\n')
@@ -955,8 +1039,10 @@ class TextCommitRenderer:
           % (text, d.base_rev, d.base_path, is_dir))
 
   def _render_diffs(self, diffs, section_header):
-    """Render diffs. Write the SECTION_HEADER iff there are actually
+    """Render diffs. Write the SECTION_HEADER if there are actually
     any diffs to render."""
+    if not diffs:
+      return
     w = self.output.write
     section_header_printed = False
 
@@ -968,11 +1054,14 @@ class TextCommitRenderer:
         section_header_printed = True
       if diff.kind == 'D':
         w('\nDeleted: %s\n' % diff.base_path)
+      elif diff.kind == 'A':
+        w('\nAdded: %s\n' % diff.path)
       elif diff.kind == 'C':
         w('\nCopied: %s (from r%d, %s)\n'
           % (diff.path, diff.base_rev, diff.base_path))
-      elif diff.kind == 'A':
-        w('\nAdded: %s\n' % diff.path)
+      elif diff.kind == 'W':
+        w('\nCopied and modified: %s (from r%d, %s)\n'
+          % (diff.path, diff.base_rev, diff.base_path))
       else:
         # kind == 'M'
         w('\nModified: %s\n' % diff.path)
@@ -989,7 +1078,7 @@ class TextCommitRenderer:
         if diff.singular:
           w('Binary file. No diff available.\n')
         else:
-          w('Binary files. No diff available.\n')
+          w('Binary file (source and/or target). No diff available.\n')
         continue
 
       for line in diff.content:
@@ -1013,8 +1102,10 @@ class Repository:
 
     self.author = self.get_rev_prop(svn.core.SVN_PROP_REVISION_AUTHOR)
 
-  def get_rev_prop(self, propname):
-    return svn.fs.revision_prop(self.fs_ptr, self.rev, propname, self.pool)
+  def get_rev_prop(self, propname, rev = None):
+    if not rev:
+      rev = self.rev
+    return svn.fs.revision_prop(self.fs_ptr, rev, propname, self.pool)
 
   def get_root(self, rev):
     try:
@@ -1131,7 +1222,8 @@ class Config:
         # then just return the value unchanged.
         setattr(self.maps, optname,
                 lambda value,
-                       sect=getattr(self, sectname): getattr(sect, value,
+                       sect=getattr(self, sectname): getattr(sect,
+                                                             value.lower(),
                                                              value))
         # mark for removal when all optnames are done
         if sectname not in mapsections:
