@@ -17,15 +17,24 @@ if test "$#" != 2; then
     exit 1
 fi
 
-glom=mailer-t1.current
+scripts="`dirname $0`"
+scripts="`cd $scripts && pwd`"
+
+glom=$scripts/mailer-t1.current
+orig=$scripts/mailer-t1.output
+conf=$scripts/mailer.conf
 rm -f $glom
 
+export TZ=GST
+
 youngest="`svnlook youngest $1`"
 for rev in `python -c "import string; print string.join(map(str, range(1,$youngest+1)))"`; do
-  $2 commit $1 $rev >> $glom
+  $2 commit $1 $rev $conf >> $glom
 done
 
 echo "current mailer.py output in: $glom"
 
-echo diff -q mailer-t1.output $glom
-diff -q mailer-t1.output $glom && echo "SUCCESS: no differences detected"
+dos2unix $glom
+
+echo diff -q $orig $glom
+diff -q $orig $glom && echo "SUCCESS: no differences detected"
